#!/usr/bin/perl

# Copyright IBM Corp. 2007
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
# @(#)94       1.35  src/kva/config/scripts/aix523/aixDataProvider.pl, fac_agents, omviosprem_dev, 200905010826 5/1/09 03:32:00

# verify `LANG=C /usr/sbin/xxx 2> /dev/null` is used throughout the program

use strict;
use POSIX qw(strftime);

################################################################################

for ($ARGV[0]) {
  # AIX/VIOS attribute groups
  if    ( /^Physical_Volumes$/        ) { Physical_Volumes()        }
  elsif ( /^Volume_Groups$/           ) { Volume_Groups()           }
  elsif ( /^Logical_Volumes$/         ) { Logical_Volumes()         }
  elsif ( /^File_Systems$/            ) { File_Systems()            }
  elsif ( /^NIM_Resources$/           ) { NIM_Resources()           }
  elsif ( /^Print_Queues$/            ) { Print_Queues()            }
  elsif ( /^Network_Interfaces$/      ) { Network_Interfaces()      }
  elsif ( /^Network_Adapters_Totals$/ ) { Network_Adapters_Totals() }
  elsif ( /^Network_Adapters_Rates$/  ) { Network_Adapters_Rates()  }
  elsif ( /^Defined_Users$/           ) { Defined_Users()           }
  elsif ( /^Active_Users$/            ) { Active_Users()            }
  elsif ( /^WPAR_CPU$/                ) { WPAR_CPU()                }
  elsif ( /^WPAR_Physical_Memory$/    ) { WPAR_Physical_Memory()    }
  elsif ( /^WPAR_Network$/            ) { WPAR_Network()            }
  elsif ( /^WPAR_FileSystem$/         ) { WPAR_Filesystem()         }
  elsif ( /^WPAR_Information$/        ) { WPAR_Information()        }
  elsif ( /^Devices$/                 ) { Devices()                 }
  elsif ( /^MPIO_Statistics$/         ) { MPIO_Statistics()         }
  elsif ( /^MPIO_Attributes$/         ) { MPIO_Attributes()         }
  elsif ( /^NPIV_Mappings$/           ) { NPIV_Mappings()           }
  elsif ( /^NPIV_FCP$/                ) { NPIV_FCP()                }


  # VIOS attribute groups
  elsif ( /^Storage_Mappings$/ ) { Storage_Mappings() }
  elsif ( /^Network_Mappings$/ ) { Network_Mappings() }
  elsif ( /^Security_States$/  ) { Security_States()  }
  elsif ( /^Firewall$/         ) { Firewall()         }
  elsif ( /^Capabilities$/     ) { Capabilities()     }
  elsif ( /^Shared_Ethernet_Adapter$/     ) { Shared_Ethernet_Adapter()     }
}

############################### Physical_Volumes ###############################
# Name, State, Volume_Group_Name, Number_of_Logical_Volumes                    #
# Number_of_Stale_Partitions, Size_MB, Free_MB, Used_MB, Free_Pct, Used_Pct    #
################################################################################

sub Physical_Volumes {
  foreach my $line (`LANG=C /usr/sbin/lspv 2> /dev/null`) {
    if ( $line =~ /^(\S+)/ ) {
      my $lspv_output = `LANG=C /usr/sbin/lspv $1 2> /dev/null`;
      if ( $lspv_output ne '' ) { print_pv($lspv_output) }
    }
  }
}

sub print_pv {
  my $lspv_output = shift;
  my ($pvname,$unique_id,$state,$vgname,$nLV,$nSP,$sizeMB,$freeMB,$usedMB,$freePct,$usedPct);

  if ( $lspv_output =~ /^\s*PHYSICAL VOLUME:\s*(\S+)/ ) { $pvname = $1 }
  else { $pvname = 'unknown' }

  my $odmget_command = "/usr/bin/odmget CuAt | grep -p $pvname | grep -p unique_id | grep value 2>/dev/null";
  my $odmget_output = `LANG=C $odmget_command`;
  my @odmget_output_array = split (/"/, $odmget_output);
  $unique_id = $odmget_output_array[1]; 

  if ( $lspv_output =~ /\n\s*PV STATE:\s*(\S+)/ ) { $state = $1 }
  else { $state = 'unknown' }

  if ( $lspv_output =~ /VOLUME GROUP:\s*(\S+)\s*\n/ ) { $vgname = $1 }
  else { $vgname = 'unknown' }

  if ( $lspv_output =~ /LOGICAL VOLUMES:\s*(\d+)\s*\n/ ) { $nLV = $1 }
  else { $nLV = -1 }

  if ( $lspv_output =~ /\n\s*STALE PARTITIONS:\s*(\d+)/ ) { $nSP = $1 }
  else { $nSP = -1 }

  if ( $lspv_output =~ /\n\s*TOTAL PPs:\s*\d+\s*\(\s*(\d+)\s*megabytes\s*\)/ )
       { $sizeMB = $1 }
  else { $sizeMB = -1 }

  if ( $lspv_output =~ /\n\s*FREE PPs:\s*\d+\s*\(\s*(\d+)\s*megabytes\s*\)/ )
       { $freeMB = $1 }
  else { $freeMB = -1 }

  if ( $lspv_output =~ /\n\s*USED PPs:\s*\d+\s*\(\s*(\d+)\s*megabytes\s*\)/ )
       { $usedMB = $1 }
  else { $usedMB = -1 }

  if ( ($freeMB != -1) && ($sizeMB != -1) )
       { $freePct = ($freeMB / $sizeMB) * 100 }
  else { $freePct = -1 }

  if ( ($usedMB != -1) && ($sizeMB != -1) )
       { $usedPct = ($usedMB / $sizeMB) * 100 }
  else { $usedPct = -1 }

  print "$pvname,$state,$vgname,$nLV,$nSP,$sizeMB,$freeMB,$usedMB,";
  printf "%.0f,%.0f,%s\n", $freePct, $usedPct,$unique_id;
}

################################ Volume_Groups #################################
# Name, State, Number_of_Logical_Volumes, Number_of_Physical_Volumes,          #
# Number_of_Active_Physical_Volumes, Number_of_Stale_Physical_Volumes,         #
# Size_MB, Free_MB, Used_MB, Free_Pct, Used_Pct                                #
################################################################################

sub Volume_Groups {
  foreach my $line (`LANG=C /usr/sbin/lsvg 2> /dev/null`) {
    if ( $line =~ /^(\S+)/ ) {
      my $lsvg_output = `LANG=C /usr/sbin/lsvg $1 2> /dev/null`;
      if ( $lsvg_output ne '' ) { print_vg($lsvg_output) }
    }
  }
}

sub print_vg {
  my $lsvg_output = shift;
  my ($vgname,$state,$nLV,$nPV,$nAPV,$nSPV,$sizeMB,$freeMB,$usedMB,$freePct,$usedPct);

  if ( $lsvg_output =~ /^\s*VOLUME GROUP:\s*(\S+)/ ) { $vgname = $1 }
  else { $vgname = 'unknown' }

  if ( $lsvg_output =~ /\n\s*VG STATE:\s*(\S+)/ ) { $state = $1 }
  else { $state = 'unknown' }

  if ( $lsvg_output =~ /\n\s*LVs:\s*(\d+)/ ) { $nLV = $1 }
  else { $nLV = -1 }

  if ( $lsvg_output =~ /\n\s*TOTAL PVs:\s*(\d+)/ ) { $nPV = $1 }
  else { $nPV = -1 }

  if ( $lsvg_output =~ /\n\s*ACTIVE PVs:\s*(\d+)/ ) { $nAPV = $1 }
  else { $nAPV = -1 }

  if ( $lsvg_output =~ /\n\s*STALE PVs:\s*(\d+)/ ) { $nSPV = $1 }
  else { $nSPV = -1 }

  if ( $lsvg_output =~ /TOTAL PPs:\s*\d+\s*\(\s*(\d+)\s*megabytes\s*\)\s*\n/ )
       { $sizeMB = $1 }
  else { $sizeMB = -1 }

  if ( $lsvg_output =~ /FREE PPs:\s*\d+\s*\(\s*(\d+)\s*megabytes\s*\)\s*\n/ )
       { $freeMB = $1 }
  else { $freeMB = -1 }

  if ( $lsvg_output =~ /USED PPs:\s*\d+\s*\(\s*(\d+)\s*megabytes\s*\)\s*\n/ )
       { $usedMB = $1 }
  else { $usedMB = -1 }

  if ( ($freeMB != -1 ) && ($sizeMB != -1) )
       { $freePct = ($freeMB / $sizeMB) * 100 }
  else { $freePct = -1 }

  if ( ($usedMB != -1 ) && ($sizeMB != -1) )
       { $usedPct = ($usedMB / $sizeMB) * 100 }
  else { $usedPct = -1 }

  print "$vgname,$state,$nLV,$nPV,$nAPV,$nSPV,$sizeMB,$freeMB,$usedMB,";
  printf "%.0f,%.0f\n", $freePct, $usedPct;
}

############################### Logical_Volumes ################################
# Name, State, Volume_Group_Name, Type, Mount_Point, Size_MB                   #
################################################################################

sub Logical_Volumes {
  foreach my $line1 (`LANG=C /usr/sbin/lsvg 2> /dev/null`) {
    if ( $line1 =~ /^(\S+)/ ) {
      my @lines2 = `LANG=C /usr/sbin/lsvg -l $1 2> /dev/null`;
      shift @lines2;
      foreach my $line2 (@lines2) {
        if ( $line2 =~ /^(\S+)/ ) {
          my $lslv_output = `LANG=C /usr/sbin/lslv $1 2> /dev/null`;
          if ( $lslv_output ne '' ) { print_lv($lslv_output) }
        }
      }
    }
  }
}

sub print_lv {
  my $lslv_output = shift;
  my ($lvname, $state, $vgname, $type, $mp, $sizeMB);

  if ( $lslv_output =~ /^\s*LOGICAL VOLUME:\s*(\S+)/ ) { $lvname = $1 }
  else { $lvname = 'unknown' }

  if ( $lslv_output =~ /LV STATE:\s*(\S+)\s*\n/ ) { $state = $1 }
  else { $state = 'unknown' }

  if ( $lslv_output =~ /VOLUME GROUP:\s*(\S+)\s*\n/ ) { $vgname = $1 }
  else { $vgname = 'unknown' }

  if ( $lslv_output =~ /\n\s*TYPE:\s*(\S+)/ ) { $type = $1 }
  else { $type = 'unknown' }

  if ( $lslv_output =~ /\n\s*MOUNT POINT:\s*(\S+)/ ) { $mp = $1 }
  else { $mp = 'unknown' }

  if ( $lslv_output =~ /PP SIZE:\s*(\d+)\s*megabyte\(s\)\s*\n/ ) {
    my $ppSize = $1;
    if ( $lslv_output =~ /PPs:\s*(\d+)\s*\n/ ) {
      my $nPP = $1;
      $sizeMB = $ppSize * $nPP;
    }
    else { $sizeMB = -1 }
  }
  else { $sizeMB = -1 }

  print "$lvname,$state,$vgname,$type,$mp,$sizeMB\n";
}

################################# File_Systems #################################
# Name, Mount_Point, Volume_Group_Name, Size_MB, Free_MB, Used_MB, Free_Pct,   #
# Used_Pct                                                                     #
################################################################################
sub File_Systems {
    my %hash    = ();
    my $vg      = ();
    my $command = "/usr/sbin/lsvg | /usr/sbin/lsvg -i -l 2>/dev/null";
    my $df_arg  = ();
    chomp(my @output = `LANG=C $command`);
    foreach my $line (@output) {
        if ($line =~ /^LV NAME/) {
            next;
        }
        if ($line =~ /:$/) {
            $vg = $line;
            $vg =~ s/://;
            next;
        }

        my ($lv_name, $type, $lp, $pp, $pv, $state, $mntpt) = split(/\s+/, $line);
        my $fs_name = "/dev/" . $lv_name;
        if ($mntpt !~ /N\A/) {
            $df_arg = join(" ", $df_arg, $fs_name);
        }

        $hash{$lv_name}{"type"}  = $type;
        $hash{$lv_name}{"lp"}    = $lp;
        $hash{$lv_name}{"pp"}    = $pp;
        $hash{$lv_name}{"pv"}    = $pv;
        $hash{$lv_name}{"state"} = $state;
        $hash{$lv_name}{"mntpt"} = $mntpt;
        $hash{$lv_name}{"vg"}    = $vg;
    }

    $command = "/usr/bin/df -m $df_arg 2>/dev/null";
    chomp(@output = `LANG=C $command`);
    foreach my $line (@output) {
        if ($line =~ /^Filesystem\s+MB blocks/) {
            next;
        }

        my ($fs_name, $mb_blocks, $free, $pused, $Iused, $pIused, $mntpt) = split(/\s+/, $line);
        my ($trash, $lv_name) = split("/dev/", $fs_name);

        if ( exists($hash{$lv_name}) ) {
            $hash{$lv_name}{"size"}  = sprintf("%.0f", $mb_blocks);
            $hash{$lv_name}{"free"}  = sprintf("%.0f", $free);
            $hash{$lv_name}{"used"}  = sprintf("%.0f", ($mb_blocks - $free));
            $hash{$lv_name}{"pused"} = sprintf("%.0f", $pused);
            $hash{$lv_name}{"pfree"} = sprintf("%.0f", (100 - $pused));

        }
    }

    foreach my $lv_name (keys %hash) {
        if ($hash{$lv_name}{"mntpt"} ne "N/A") {
            my $itm_mntpt = $hash{$lv_name}{"mntpt"};
            my $itm_vg    = $hash{$lv_name}{"vg"};
            my $itm_size  = $hash{$lv_name}{"size"};
            my $itm_free  = $hash{$lv_name}{"free"};
            my $itm_used  = $hash{$lv_name}{"used"};
            my $itm_pfree = $hash{$lv_name}{"pfree"};
            my $itm_pused = $hash{$lv_name}{"pused"};

            print "$lv_name,";
            print "$itm_mntpt,";
            print "$itm_vg,";
            print "$itm_size,";
            print "$itm_free,";
            print "$itm_used,";
            print "$itm_pfree,";
            print "$itm_pused\n";
        }
    }
}

################################ NIM_Resources #################################
# Name, Type, Class, State, Server, Location, Information                      #
################################################################################
sub NIM_Resources {
	my $nimtype = get_nimconfig();
	my %nim = get_nimoutput($nimtype);
	if ( $nimtype eq "master" ) {
		foreach my $object (keys %nim) {
			my $name     = $object;
			my $type     = $nim{$object}{"type"};
			my $class    = $nim{$object}{"class"};
			my $server   = " ";
			my $location = " ";
			my $state    = " ";
			my $info     = " ";

			if ( exists($nim{$object}{"info"}) ) {
				$info     = $nim{$object}{"info"};
			}

			if ($nim{$object}{"class"} eq "machines") {
				$state    = $nim{$object}{"Mstate"};
			} elsif ($nim{$object}{"class"} eq "resources") { 
				$server   = $nim{$object}{"server"};
				$location = $nim{$object}{"location"};
				$state    = $nim{$object}{"Rstate"};
			}
			print "$name,";
			print "$type,";
			print "$class,";
			print "$state,";
			print "$server,";
			print "$location,";
			print "$info\n";
		}
        
	} else {
		foreach my $object (keys %nim) {
			my $class    = $nim{$object}{"class"};
			if ($class eq "resources") {
				my $name     = $object;
				my $type     = $nim{$object}{"type"};
				my $class    = $nim{$object}{"class"};
				my $server   = " ";
                        	my $location = " ";
	                        my $state    = " ";
				my $info     = " ";

				if ( exists($nim{$object}{"info"}) ) {
					$info     = $nim{$object}{"info"};
				}

				if ($nim{$object}{"class"} eq "machines") {
	                                $state    = $nim{$object}{"Mstate"};
        	                } elsif ($nim{$object}{"class"} eq "resources") {
	                                $server   = $nim{$object}{"server"};
	                                $location = $nim{$object}{"location"};
        	                        $state    = $nim{$object}{"Rstate"};
                	        }

	                        print "$name,";
        	                print "$type,";
	                        print "$class,";
        	                print "$state,";
                	        print "$server,";
                        	print "$location,";
				print "$info\n";
			}
                }  
	}
}

#
# This subroutine runs the lsnim -l output and store the output
# in a hash of hash with in the following layout:
# lsnim -l output
#       object1:
#               attribute1 = value1
#               attribute2 = value2
#       object2:
#               attribute1 = value1
#               attribute2 = value2
#
# Hash of Hash
# { object1 => { attribute1 => value1,
#                attribute2 => value2 },
#   object2 => { attribute1 => value1,
#                attribute2 => value2 }}
#
sub get_nimoutput {
	my ($nimtype) = @_;
	my %hash = ();
	my $command    = "/usr/sbin/lsnim -l 2>/dev/null";

	#my %niminfo    = get_niminfo();
        #my $nim_master = $niminfo{"NIM_MASTER_HOSTNAME"};
	#if ($nim_master ne "" ) {
	#	`/etc/ping -c1 $nim_master 2>&1 >/dev/null`;
	#	if ($? != 0) {
	#		return %hash;
	#	}
	#}

	if ( $nimtype eq "client") {
		$command = "/usr/sbin/nimclient -l -l 2>/dev/null";
	}

        my @server_output = `LANG=C $command`;
        my $count = @server_output;

        for (my $i = 0; $i < $count; $i++) {
                if ($server_output[$i] =~ /:$/) {
                        my ($object) = split(/:/, $server_output[$i]);
                        $i++;
                        while ($i < $count) {
                                if ( !($server_output[$i] =~ /:$/) ) {
                                        my ($attribute, $value) = split(/=/, $server_output[$i]);
                                        if ( defined($attribute) && defined($value) ) {
                                                $attribute =~ s/^\s+//;
                                                $attribute =~ s/\s+$//;
                                                chomp($attribute);
                                                $value =~ s/^\s+//;
                                                chomp($value);
                                                $hash{$object}{$attribute} = $value;
                                        }
                                        $i++;
                                } else {
                                        $i--;
                                        last;
                                }
                        }
                }
        }
        return %hash;
}

# Determine if bos.sysmgt.nim.master is installed
# Return 0 if master fileset is installed
sub has_nim_master {
        my $rc = 1;
        my $command = "/usr/bin/lslpp -L bos.sysmgt.nim.master 2>/dev/null";
        my @output = `LANG=C $command`;
        if ( $? == 0 ) {
                $rc = 0;
        }

        return $rc;
}

# Determine if bos.sysmgt.nim.client is installed
# Return 0 if client fileset is installed
sub has_nim_client {
        my $rc = 1;
        my $command = "/usr/bin/lslpp -L bos.sysmgt.nim.client 2>/dev/null";
        my @output = `LANG=C $command`;
        if ( $? == 0 ) {
                $rc = 0;
        }

        return $rc;
}

# Determine if NIM is configured as a NIM master 
# Return 0 if machine is configured as a NIM master.
sub is_configured {
	my $rc = 1;
	my $command = "ODMDIR=/etc/objrepos /usr/bin/odmget -q \"name=master\" nim_object 2>/dev/null";
	chomp(my @output = `LANG=C $command`);
	if ( @output > 0 ) {
                $rc = 0;
        }

        return $rc;
}

# Return string "master" if machine is a NIM master
# Return string "client" if machine is a NIM client
# Return string "no" if machine is not participating in the NIM environment
sub get_nimconfig {
	my $nim = "no";
	my %niminfo = get_niminfo();

	if ( (has_nim_master() == 0) && 
             (is_configured() == 0)  && 
             ($niminfo{"NIM_CONFIGURATION"} eq "master") 
           ) {
		$nim = "master";
	} elsif ( (has_nim_client() == 0)                           &&
                  ( ($niminfo{"NIM_CONFIGURATION"} eq "standalone") ||
                    ($niminfo{"NIM_CONFIGURATION"} eq "diskless")   ||
		    ($niminfo{"NIM_CONFIGURATION"} eq "dataless") ) ) {
		$nim = "client";
	}
	
	return $nim;
}

# Get the content of the /etc/niminfo the determine
# whether machine is configure as a NIM master or a
# NIM client.
sub get_niminfo {
	my %hash = ();
	my $infile = "/etc/niminfo";

	if ( (-e $infile) && !(-z $infile) ) {
	        open(INFILE, "<$infile");
        	while(<INFILE>) {
	                if ($_ =~ /^#/) { next; }
	
			if ($_ =~ /^export/) {
				my ($export, $content)  = split(/\s+/, $_);
				my ($attribute, $value) = split(/=/, $content);
				$hash{$attribute} = $value;
			}
		}
		close(INFILE);
	}

	return %hash;
}

################################# Print_Queues #################################
# Name, Device, State, Description                                             #
################################################################################
# ITM function to print print queue information
# Get the hash of queue from get_queue() and
# get description from get_pioslvp()
# Header format:
# Name,State,Device,Description
sub Print_Queues {
        my %queue   = get_queue();
        my %pioslvp = get_pioslvp();
        my %enq     = get_enq();

	foreach my $queue_name (keys %queue) {
                my ($name, $state, $dev_state, $device, $description) = ("", "", "", "", "");
	        $name        = $queue_name;
                $description = $pioslvp{$queue_name}{"description"};
               	$device      = $queue{$queue_name}{"device"};

		if ( exists($queue{$queue_name}{"up"}) ) {
                        $dev_state = $queue{$queue_name}{"up"};
               	}

	        if ( exists($enq{$queue_name}{"status"}) ) {
                        $state = $enq{$queue_name}{"status"};
               	}

	        print "$name,";
                print "$device,";
               	print "$state,";
	        print "$description\n";
	}
}

# Return a hash of print queue - no description
sub get_queue {
	my %hash  = ();
	my @queue = get_lsallq();
	foreach my $queue (@queue) {
		my $command = "/usr/bin/lsque -q $queue 2>/dev/null";
		chomp(my @output = `LANG=C $command`);
		my $count = @output;
		my $name = "";
		my %queue = ();
		for (my $i = 0; $i < $count; $i++) {
			if ( ($i == 0) && ($output[$i] =~ /:$/) ) {
				$output[$i] =~ s/://g;
				$name = $output[$i];
				
			} else {
				my ($blank, $attribute, $separator, $value) = split(/\s+/, $output[$i]);
				$queue{$attribute} = $value;
			}
		}
		$hash{$name} = \%queue;

	}

        return %hash;
}

# Return a hash of print queue with its description - no queue device
sub get_pioslvp {
	my %hash = ();
	my $command = "/usr/lib/lpd/pio/etc/piolsvp -p 2>/dev/null";
        chomp(my @output = `LANG=C $command`);
	foreach my $line (@output) {
		if ($line !~ /^#/) {
			my @result      = split(/\s+/, $line);
			my $name        = $result[1];
			my $device      = $result[2];
			my @description = @result[3..$#result];
			my $description = join(" ", @description);
			$hash{$name}{"device"} = $device;
			$hash{$name}{"description"} = $description;
		}
	}

        return %hash;
}

# Return the enq
sub get_enq {
        my %hash = ();
        my $command = "/usr/bin/enq -esAW 2>/dev/null";
        chomp(my @output = `LANG=C $command`);
        my $count = @output;
        for (my $i = 2; $i < $count; $i++) {
                my ($queue, $dev, $status) = split(/\s+/, $output[$i]);
                my %queue        = ();
                $queue{"dev"}    = $dev;
                $queue{"status"} = $status;
                $hash{$queue}    = \%queue;
        }

        return %hash;
}

# Get all print queue
# Return array of all queue
sub get_lsallq {
	my $command = "/usr/bin/lsallq 2>/dev/null";
	chomp(my @queue = `LANG=C $command`);

	return @queue;
}

############################## Devices #########################################
# Name, Parent, Type, State, Class                                             #
################################################################################
# Main function to get Network Interface metrics
sub Devices {
	my $command = "";

	my $delimiter = ":";
	$command = "/usr/sbin/lsdev -C -F 'name:parent:type:status:class' 2>/dev/null";

	my @output = `LANG=C $command`;
	foreach my $output (@output) 
	{
		chomp($output);
		my ($name, $parent, $type, $status, $class) = split(/$delimiter/, $output);
		#Some devices have comma in their names. Hence replace it with
		#underscore so that the dataprovider dont get confused
		$name =~ s/,/_/g;
		$parent =~ s/,/_/g;
		$type =~ s/,/_/g;
		$status =~ s/,/_/g;
		$class =~ s/,/_/g;

        if ($name =~ /ent/) {
                        my @templines = execAndRead($name);
                        foreach(@templines) {
                                if ($_ =~ /LAN State/) {
                                        my $statline = $_;
                                        if ( !($statline =~ /Operational/) ) {
                                                $status = "Stopped";
                                        }
                                }
                                elsif ($_ =~ /Link Stat|Port Operational State/) {
                                        my $statline = $_;
                                        if ( !($statline =~ /Up|up/) ) {
                                                $status = "Stopped";
                                        }
                                }
                        }
                }


		print "$name,$parent,$type,$status,$class\n";
	}
}

############################## Network_Interfaces ##############################
# Name, State, IP_Address, MTU, Mask, Domain, Gateway, Nameserver              #
################################################################################
# Main function to get Network Interface metrics
sub Network_Interfaces {
	my %lsdev = get_lsdev();

	my %namerslv   = get_namerslv();
	my $domain     = $namerslv{"domain"};
	my $nameserver = (length($namerslv{"nameserver"}) > 0) ? $namerslv{"nameserver"} : " ";

	my %inet0      = get_lsattr("inet0");
	my $hostname   = $inet0{"hostname"};

	# split the following formated output: net,-hopcount,0,,0,9.3.63.1
	my @route      = split(/,/, $inet0{"route"});
	my $count      = @route;
        #my $gateway    = $route[$count - 1];

        # Defect#92979. 
        my $cmd = "/usr/bin/netstat -r -n| tr -s ' '" ;
        my @output = `LANG=C $cmd`;
        my $gateway;

        foreach my $output (@output)
        {
          if(($output =~/default/))
          {
                (my $junk, $gateway) = split(/ /,$output);
                last;
          }
        }


	my %ifconfig   = get_ifconfig();

	foreach my $device (keys %lsdev) {
		if ( ( $lsdev{$device}{"type"} eq "en") && ($ifconfig{$device}{"state"} eq "up") ) {
			my %lsattr  = get_lsattr($device);
			my $state   = $ifconfig{$device}{"state"};
			my $mtu     = $lsattr{"mtu"};
			my $name    = $lsattr{"name"};
			my $netaddr = $ifconfig{$device}{"inet"};
			my $netmask = $lsattr{"netmask"};

			print "$name,";
			print "$state,";
			print "$netaddr,";
			print "$mtu,";
			print "$netmask,";
			print "$domain,";
			print "$gateway,";
			print "$nameserver\n";
		}
	}
}

sub get_namerslv {
	my $command = "/usr/sbin/namerslv -s 2>/dev/null";
	my @output = `LANG=C $command`;
	my %hash = ();

	foreach my $output (@output) {
		chomp($output);
		my ($attribute, $value) = split(/\s+/, $output);
		if (exists($hash{$attribute})) {
			$value = join(":", $hash{$attribute}, $value);
		}
		$hash{$attribute} = $value;
	}

	return %hash;
}

sub get_lsattr {
	my ($adapter) = @_;
	my %hash = ();

        my $command = "/usr/sbin/lsattr -F \"attribute:value\" -l $adapter 2>/dev/null";
        my @output = `LANG=C $command`;

	# The 'netaddr' are not valid for sea adapters
	$hash{"name"} = $adapter;
	if (@output > 1) {
	        foreach my $output (@output) {
        	        chomp($output);
                  	my ($attribute, $value) = split(/:/, $output);
                        $hash{$attribute} = $value;
		}
	}

        return %hash;
}

#
# Process the output from ifconfig -a and generate a hash from it.
#
# Example of the ifconfig -a output:
#en4: flags=1e080863,80<UP,BROADCAST,NOTRAILERS,RUNNING,SIMPLEX,MULTICAST,GROUPRT,64BIT,CHECKSUM_OFFLOAD(ACTIVE),CHAIN>
#        inet 9.3.94.136 netmask 0xfffffe00 broadcast 9.3.95.255
#         tcp_sendspace 131072 tcp_recvspace 65536
#en5: flags=1e080863,80<UP,BROADCAST,NOTRAILERS,RUNNING,SIMPLEX,MULTICAST,GROUPRT,64BIT,CHECKSUM_OFFLOAD(ACTIVE),CHAIN>
#        inet 10.1.1.2 netmask 0xff000000 broadcast 10.255.255.255
#         tcp_sendspace 131072 tcp_recvspace 65536
#en1: flags=5e080862,c0<BROADCAST,NOTRAILERS,RUNNING,SIMPLEX,MULTICAST,GROUPRT,64BIT,CHECKSUM_OFFLOAD(ACTIVE),PSEG,LARGESEND,CHAIN>
#         tcp_sendspace 131072 tcp_recvspace 65536
#lo0: flags=e08084b<UP,BROADCAST,LOOPBACK,RUNNING,SIMPLEX,MULTICAST,GROUPRT,64BIT>
#        inet 127.0.0.1 netmask 0xff000000 broadcast 127.255.255.255
#        inet6 ::1/0
#         tcp_sendspace 131072 tcp_recvspace 131072 rfc1323 1
sub get_ifconfig {
	my $ifconfig_cmd = "/etc/ifconfig -a 2>/dev/null";
	chomp(my @ifconfig     = `LANG=C $ifconfig_cmd`);
	my $ifconfig_cnt = @ifconfig;
	my %hash  = ();
	my $state = ();
	for (my $i = 0; $i < $ifconfig_cnt; $i++) {
		if ($ifconfig[$i] =~ /: flags=/) {
			my ($data,      $flags_string)   = split(/</, $ifconfig[$i]);
			my ($device,    $flags_settings) = split(/:/, $data);
			my ($attribute, $flags_value)    = split(/=/, $flags_settings);
			my ($flags,	$chain)		 = split(/,/, $flags_value);

			$hash{$device}{"flags"} = $flags;
			$hash{$device}{"chain"} = $chain;

			$flags_string =~ s/>$//g;
			my @flags = split(/,/, $flags_string);
			$state = "down";
			foreach my $val (@flags) {
				if ($val eq "UP") {
					$state = "up";
					last;
				}
			}
			$hash{$device}{"state"} = $state;

			# Get next line
			$i++;
			if ($ifconfig[$i] =~ /\s+inet /) {
				my @info     = split(/\s+/, $ifconfig[$i]);
				my $info_cnt = @info;
				for (my $j = 1; $j < $info_cnt; $j++) {
					my $attr = $info[$j];
					my $value = $info[$j+1];
					$hash{$device}{$attr} = $value;
					$j++;
				}
				$i++;
			}

			if ($ifconfig[$i] =~ /\s+tcp_sendspace /) {
                                my @info     = split(/\s+/, $ifconfig[$i]);
                                my $info_cnt = @info;
                                for (my $j = 1; $j < $info_cnt; $j++) {
					my $attr = $info[$j];
					my $value = $info[$j+1];
					$hash{$device}{$attr} = $value; 
                                        $j++;
                                }
				$i++;
                        } 

			if ($ifconfig[$i] =~ /: flags=/) {
				$i--;
			}
		} else {
			next;
		}
	}

	return %hash;
}

########################### Network_Adapters_Totals ############################
# Name, Parent, Type, Bytes_Sent, Pkts_Sent, Pkts_Sent_Error,                  #
# Sent_Pkts_Dropped, Broadcast_Pkts_Sent, Multicast_Pkts_Sent,                 #
# Sent_Interrupts, Bytes_Recvd, Pkts_Recvd, Pkts_Recv_Error, Bad_Pkts_Recvd,   #
# Recv_Rkts_Dropped, Broadcast_Pkts_Recvd, Multicast_Pkts_Recvd,               #
# Recv_Interrupts, TransmitsQ, Max_TransmitsQ, Qoverflow, Real_Pkts_Recvd,     #
# Real_Pkts_Bridged, Real_Pkts_Consumed, Real_Pkts_Fragmented, Real_Pkts_Sent, #
# Real_Pkts_Dropped, Virtual_Pkts_Recvd, Virtual_Pkts_Bridged,                 #
# Virtual_Pkts_Consumed, Virtual_Pkts_Fragmented, Virtual_Pkts_Sent,           #
# Virtual_Pkts_Dropped, Output_Pkts_Generated, Output_Pkts_Dropped,            #
# Output_Pkts_Failures, Mem_Alloc_Failures, ICMP_Error_Pkts_Sent,              #
# Non_IP_Pkts_Larger_Than_MTU, ThreadQ_Overflow_Pkts, HA_Keep_Alive_Pkts,      #
# HA_Recovery_Pkts, HA_Notify_Pkts, HA_Limbo_Pkts, HA_State, HA_Bridge_Mode,   #
# Times_Primary, Times_Backup, HA_Mode, Priority                               #
################################################################################
sub Network_Adapters_Totals {
	my %data = getDevData();
	printAttribute(\%data);
}

sub getDevData {
	my %lines;
	my %data;
	my @devs = getDevList();
	# first read data for all device then parse ouptut
  foreach my $arg (@devs) {
   	my @templines = execAndRead($arg);
    $lines{$arg} = [ @templines ];
  }
  for my $arg (keys %lines) {
    my @templines = @{$lines{$arg}};
    my %tempdata = processDevice(@templines);
    for my $dev (keys %tempdata) {
      $data{$dev} = $tempdata{$dev};
    }
	}
	return %data;
}

sub getDevList {
        my @devlist = ();
	my $cmd = "LANG=C /usr/sbin/lsdev 2>/dev/null|";
        open(OUT, $cmd) || die "Failed: $!\n";
        while(<OUT>) {
          if($_ =~ m/^(ent\d+)\s/) {
            push(@devlist, $1);
          }
        }
        return @devlist;
}

# arg0 - device name
sub execAndRead {
  my $device = shift;
	my $cmd = 'LANG=C entstat -d ' . $device . " 2>/dev/null|";
#	$cmd = 'cat entstat.out.12|';
	open(OUT, $cmd) || die "Failed: $!\n";
	my @lines = <OUT>;
	close(OUT);
	return @lines;
}

# arg0 - array of lines of text for that device
sub processDevice {
  my @text = @_;
	my ($vssSection, $otherSection, $haSection);
	my %devhash = ();
	my $parent = "none";
	my $etherchannel = "";
	my $vssSection = 0;
	my $otherSection = 0;
	my $haSection = 0;
	my $statPrefix = "Real_";
	my $curname = "";
  foreach my $i (@text) {
		if ($i =~ m/ETHERNET STATISTICS\W+\(([\w\d]+)\)/) {
			$curname = $1;
			$devhash{$1}{'Name'} = $1;
			$devhash{$1}{'Parent'} = $parent;
			if ($parent eq "none") {
				$parent = $1;
			}
			else {
				$haSection = 0;
			}
		}
		if ($i =~ m/Device Type: ([^\n]+)/) {
			my $type = $1;
			$devhash{$curname}{'Type'} = $type;
			if ($type =~ m/Virtual/) {
			}
			else {
				if ($etherchannel ne "") {
					$devhash{$curname}{'Parent'} = $etherchannel;
				}
			}
			if ($type eq "EtherChannel") {
				$etherchannel = $curname;
			}
		}
		# Here I want to determine whether I'm in the section titled
		#		Virtual Side Statistics/Other Statistics/High Availability
		if ($i =~ m/^Virtual Side Statistics:/) {
			$statPrefix   = 'Virtual_';
			$vssSection   = 1;
			$otherSection = 0;
			$haSection 		= 0;
		}
		if ($i =~ m/^Other Statistics:/) {
			$vssSection   = 0;
			$otherSection = 1;
			$haSection 		= 0;
		}
		if ($i =~ m/^High Availability/) {
			$vssSection   = 0;
			$otherSection = 0;
			$haSection 		= 1;
		}

		if ($i =~ m/^\s*Packets received:\s*(\d+)/) {
			$devhash{$curname}{$statPrefix . 'Pkts_Recvd'} = $1;
		}
		if ($i =~ m/^\s*Packets bridged:\s*(\d+)/) {
			$devhash{$curname}{$statPrefix . 'Pkts_Bridged'} = $1;
		}
		if ($i =~ m/^\s*Packets consumed:\s*(\d+)/) {
			$devhash{$curname}{$statPrefix . 'Pkts_Consumed'} = $1;
		}
		if ($i =~ m/^\s*Packets fragmented:\s*(\d+)/) {
			$devhash{$curname}{$statPrefix . 'Pkts_Fragmented'} = $1;
		}
		if ($i =~ m/^\s*Packets transmitted:\s*(\d+)/) {
			$devhash{$curname}{$statPrefix . 'Pkts_Sent'} = $1;
		}
		if ($i =~ m/^\s*Packets dropped:\s*(\d+)/) {
			$devhash{$curname}{$statPrefix . 'Pkts_Dropped'} = $1;
		}

		if ($otherSection == 1) {
			if ($i =~ m/^\s*Output packets generated:\s*(\d+)/) {
				$devhash{$curname}{'Output_Pkts_Generated'} = $1;
			}
			if ($i =~ m/^\s*Output packets dropped:\s*(\d+)/) {
				$devhash{$curname}{'Output_Pkts_Dropped'} = $1;
			}
			if ($i =~ m/^\s*Device output failures:\s*(\d+)/) {
				$devhash{$curname}{'Output_Pkts_Failures'} = $1;
			}
			if ($i =~ m/^\s*Memory allocation failures:\s*(\d+)/) {
				$devhash{$curname}{'Mem_Alloc_Failures'} = $1;
			}
			if ($i =~ m/^\s*ICMP error packets sent:\s*(\d+)/) {
				$devhash{$curname}{'ICMP_Error_Pkts_Sent'} = $1;
			}
			if ($i =~ m/^\s*Non IP packets larger than MTU:\s*(\d+)/) {
				$devhash{$curname}{'Non_IP_Pkts_Larger_Than_MTU'} = $1;
			}
			if ($i =~ m/^\s*Thread queue overflow packets:\s*(\d+)/) {
				$devhash{$curname}{'ThreadQ_Overflow_Pkts'} = $1;
			}
		}

		if ($haSection == 1) {
			if ($i =~ m/^\s*Keep-Alive Packets:\s*(\d+)/) {
				$devhash{$curname}{'HA_Keep_Alive_Pkts'} = $1;
			}
			if ($i =~ m/^\s*Recovery Packets:\s*(\d+)/) {
				$devhash{$curname}{'HA_Recovery_Pkts'} = $1;
			}
			if ($i =~ m/^\s*Notify Packets\s*:\s*(\d+)/) {
				$devhash{$curname}{'HA_Notify_Pkts'} = $1;
			}
			if ($i =~ m/^\s*Limbo Packets:\s*(\d+)/) {
				$devhash{$curname}{'HA_Limbo_Pkts'} = $1;
			}
			if ($i =~ m/^\s*State:\s*([\d\w]+)/) {
				$devhash{$curname}{'HA_State'} = $1;
			}
			if ($i =~ m/^\s*Bridge Mode:\s*([\d\w]+)/) {
				$devhash{$curname}{'HA_Bridge_Mode'} = $1;
			}
			if ($i =~ m/^\s*Number of Times Server became Backup:\s*(\d+)/) {
				$devhash{$curname}{'Times_Backup'} = $1;
			}
			if ($i =~ m/^\s*Number of Times Server became Primary:\s*(\d+)/) {
				$devhash{$curname}{'Times_Primary'} = $1;
			}
			if ($i =~ m/^\s*High Availability Mode:\s*([\d\w]+)/) {
				$devhash{$curname}{'HA_Mode'} = $1;
			}
			if ($i =~ m/^\s*Priority:\s*(\d+)/) {
				$devhash{$curname}{'Priority'} = $1;
			}
		}

		if ($i =~ m/^Packets:\s*(\d+)\s*Packets:\s*(\d+)/) {
			$devhash{$curname}{'Pkts_Sent'} = $1;
			$devhash{$curname}{'Pkts_Recvd'} = $2;
		}
		if ($i =~ m/^Bytes:\s*(\d+)\s*Bytes:\s*(\d+)/) {
			$devhash{$curname}{'Bytes_Sent'} = $1;
			$devhash{$curname}{'Bytes_Recvd'} = $2;
		}
		if ($i =~ m/^Interrupts:\s*(\d+)\s*Interrupts:\s*(\d+)/) {
			$devhash{$curname}{'Sent_Interrupts'} = $1;
			$devhash{$curname}{'Recv_Interrupts'} = $2;
		}
		if ($i =~ m/^Transmit Errors:\s*(\d+)\s*Receive Errors:\s*(\d+)/) {
			$devhash{$curname}{'Pkts_Sent_Error'} = $1;
			$devhash{$curname}{'Pkts_Recv_Error'} = $2;
		}
		if ($i =~ m/^Packets Dropped:\s*(\d+)\s*Packets Dropped:\s*(\d+)/) {
			$devhash{$curname}{'Sent_Pkts_Dropped'} = $1;
			$devhash{$curname}{'Recv_Pkts_Dropped'} = $2;
		}
		if ($i =~ m/^Broadcast Packets:\s*(\d+)\s*Broadcast Packets:\s*(\d+)/) {
			$devhash{$curname}{'Broadcast_Pkts_Sent'} = $1;
			$devhash{$curname}{'Broadcast_Pkts_Recvd'} = $2;
		}
		if ($i =~ m/^Multicast Packets:\s*(\d+)\s*Multicast Packets:\s*(\d+)/) {
			$devhash{$curname}{'Multicast_Pkts_Sent'} = $1;
			$devhash{$curname}{'Multicast_Pkts_Recvd'} = $2;
		}
		if ($i =~ m/^\s*Bad Packets:\s*(\d+)/) {
                    $devhash{$curname}{'Bad_Pkts_Recvd'} = $1;
		}
		if ($i =~ m/^Max Packets on S\/W Transmit Queue:\s*(\d+)/) {
			$devhash{$curname}{'Max_TransmitsQ'} = $1;
		}
		if ($i =~ m/^S\/W Transmit Queue Overflow:\s*(\d+)/) {
			$devhash{$curname}{'Qoverflow'} = $1;
		}
		if ($i =~ m/^Current .* Transmit Queue Length:\s*(\d+)/) {
			$devhash{$curname}{'TransmitsQ'} = $1;
		}
		if ($i =~ m/^\s*Priority:\s*(\d+)\s*Active:\s*(\w+)/) {
			if ($devhash{$parent}{'HA_Bridge_Mode'} eq "") {
				$devhash{$parent}{'Priority'} = $1;
			}
			if ($devhash{$parent}{'Type'} eq "Shared Ethernet Adapter") {
			  if($devhash{$parent}{'HA_Mode'} eq "") {
        		    my $cmd2 = "LANG=C /usr/sbin/lsattr -El " . $parent . " 2>/dev/null|";
        		    open(OUT, $cmd2) || die "Failed: $!\n";
        		    while(<OUT>) {
          			if($_ =~ m/^\s*ha_mode\s*(\w+)/) {
					$devhash{$parent}{'HA_Mode'} = $1;
				}
			    }
			  }
			}
		}
	}
	return %devhash;
}

# arg 0 - hash of device data
sub printAttribute {
	my %tempdata = %{$_[0]};
	my @adapterAttrib = ('Name', 'Parent', 'Type', 'Bytes_Sent', 'Pkts_Sent', 'Pkts_Sent_Error',
			     'Sent_Pkts_Dropped', 'Broadcast_Pkts_Sent', 'Multicast_Pkts_Sent',
			     'Sent_Interrupts', 'Bytes_Recvd', 'Pkts_Recvd', 'Pkts_Recv_Error', 
			     'Bad_Pkts_Recvd', 'Recv_Pkts_Dropped', 'Broadcast_Pkts_Recvd',
			     'Multicast_Pkts_Recvd', 'Recv_Interrupts', 'TransmitsQ', 'Max_TransmitsQ',
			     'Qoverflow', 'Real_Pkts_Recvd', 'Real_Pkts_Bridged', 'Real_Pkts_Consumed',
			     'Real_Pkts_Fragmented', 'Real_Pkts_Sent', 'Real_Pkts_Dropped', 
			     'Virtual_Pkts_Recvd', 'Virtual_Pkts_Bridged', 'Virtual_Pkts_Consumed',
			     'Virtual_Pkts_Fragmented', 'Virtual_Pkts_Sent', 'Virtual_Pkts_Dropped',
			     'Output_Pkts_Generated', 'Output_Pkts_Dropped', 'Output_Pkts_Failures',
			     'Mem_Alloc_Failures', 'ICMP_Error_Pkts_Sent', 'Non_IP_Pkts_Larger_Than_MTU',
			     'ThreadQ_Overflow_Pkts', 'HA_Keep_Alive_Pkts', 'HA_Recovery_Pkts',
			     'HA_Notify_Pkts', 'HA_Limbo_Pkts', 'HA_State', 'HA_Bridge_Mode', 
			     'Times_Primary', 'Times_Backup', 'HA_Mode', 'Priority');
	for my $i (keys %tempdata) {
		if($tempdata{$i}{'Parent'} eq 'none') { 
		    foreach my $attrib (@adapterAttrib) {
			print $tempdata{$i}{$attrib} . ",";
		    }
                    print "\n";
		    for my $j (keys %tempdata) {
			if($tempdata{$j}{'Parent'} eq $i) {
		    	    foreach my $attrib (@adapterAttrib) {
			        print $tempdata{$j}{$attrib} . ",";
			    }
                            print "\n";
		    	    for my $k (keys %tempdata) {
				if($tempdata{$k}{'Parent'} eq $j) {
		    	    	    foreach my $attrib (@adapterAttrib) {
			        	print $tempdata{$k}{$attrib} . ",";
			            }
                                    print "\n";
				}
			    }
			}
		    }
		}
	}
}

############################ Network_Adapters_Rates ############################
# Name, Parent, Type, Bytes_Sent_Per_Sec, Pkts_Sent_Per_Sec,                   #
# Pkts_Sent_Errors_Per_Sec, Sent_Pkts_Dropped_Per_Sec,                         #
# Broadcast_Pkts_Sent_Per_Sec, Multicast_Pkts_Sent_Per_Sec,                    #
# Sent_Interrupts_Per_Sec, Bytes_Recvd_Per_Sec, Pkts_Recvd_Per_Sec,            #
# Pkts_Recv_Errors_Per_Sec, Bad_Pkts_Recvd_Per_Sec, Rate_recv_pkts_dropped,    #
# Broadcast_Pkts_Recvd_Per_Sec, Multicast_Pkts_Recvd_Per_Sec,                  #
# Recv_Interrupts_Per_Sec, TransmitsQ, Max_TransmitsQ, Qoverflow,              #
# Real_Pkts_Recvd, Real_Pkts_Bridged, Real_Pkts_Consumed,                      #
# Real_Pkts_Fragmented, Real_Pkts_Sent, Real_Pkts_Dropped, Virtual_Pkts_Recvd, #
# Virtual_Pkts_Bridged, Virtual_Pkts_Consumed, Virtual_Pkts_Fragmented,        #
# Virtual_Pkts_Sent, Virtual_Pkts_Dropped, Output_Pkts_Generated_Per_Sec,      #
# Output_Pkts_Dropped_Per_Sec, Output_Pkts_Failures_Per_Sec,                   #
# Mem_Alloc_Failures, ICMP_Error_Pkts_sent, Non_IP_Pkts_Larger_Than_MTU,       #
# ThreadQ_Overflow_Pkts, HA_Keep_Alive_Pkts_Per_Sec, HA_Recovery_Pkts_Per_Sec, #
# HA_Notify_Pkts_Per_Sec, HA_Limbo_Pkts_Per_Sec, HA_State, HA_Bridge_Mode,     #
# Times_Primary_Per_Sec, Time_Backup_Per_Sec, HA_Mode, Priority                #
################################################################################
sub Network_Adapters_Rates {
	my %firstPassData  = getDevData();
	# wait
	sleep(10);
	my %secondPassData = getDevData();
	my %results = compareAttribs(\%firstPassData, \%secondPassData, 10);
	printAttribute(\%results);
}

# simple function to test membership in an array
# arg0 - the key that you are looking for
# arg1 - the array you are testing
sub member {
  my $key = $_[0];
  my @group = @{$_[1]};

  for my $i (@group) {
    if ($key eq $i) {
      return 1;
    }
  }
  return 0;
}

sub compareAttribs {
	my %olddata = %{$_[0]};
	my %newdata = %{$_[1]};
	my $seconds = $_[2];

	my %resultData;

	my @adapterAttrib = ('Name', 'Parent', 'Type', 'HA_Mode', 'HA_State',
			     'Priority', 'HA_Bridge_Mode');

	# we assume that olddata and newdata have all the same adapters
	for my $dev (keys %olddata) {
		for my $key (keys %{$olddata{$dev}}) {
			if (!member ($key, \@adapterAttrib)) {
				# if it is not a member, we want the rate
				$resultData{$dev}{$key} = (($newdata{$dev}{$key} - $olddata{$dev}{$key}) / $seconds);
				if($resultData{$dev}{$key} < 0) {
					$resultData{$dev}{$key} = ($newdata{$dev}{$key} / $seconds);
				}
			}
			else {
				# if not, just stick the metric in the results as is
				$resultData{$dev}{$key} = $olddata{$dev}{$key};
			}
		}
	}
	return %resultData;
}

################################ Defined_Users #################################
# User_Name, Roles, Account_Locked, Expires, Loginretries                      #
################################################################################
sub Defined_Users {
        my %hash = get_lsuser();

        foreach my $user (keys %hash) {
                my $name = $user;
                my $loginretries = $hash{$user}{"loginretries"};
                my $account_locked = $hash{$user}{"account_locked"};
                my $expires = $hash{$user}{"expires"};
                my $roles = (length($hash{$user}{"roles"}) > 0) ? $hash{$user}{"roles"} : " ";

		if ($expires eq "0101000070") {
			$expires = "disabled";
		} elsif ( ($expires eq "0") || ($expires == 0) ) {
			$expires = "never";
		} else {
			my ($month, $day, $hour, $minute, $year) = ( $expires =~ /^(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/ );
			$expires = strftime("%c", "00", $minute, $hour, $day, $month-1, $year+=100);
		}

                print "$name,";
                print "$roles,";
                print "$account_locked,";
                print "$expires,";
                print "$loginretries\n";
        }
}

sub get_lsuser() {
        my %hash    = ();
        my $delimiter = ":";
        my $command = "/usr/sbin/lsuser -R files -c ALL 2>/dev/null";
        chomp(my @output = `LANG=C $command`);
        my @attribute;
        my @value;

        foreach my $output (@output) {
                if ($output =~ /^#/) {
                        @attribute = split(/$delimiter/, $output);
                        next;
                }

                if ($output !~ /^#/) {
                        @value = split(/$delimiter/, $output);
                        my $count = @attribute;
                        for (my $i = 1; $i < $count; $i++) {
                                $hash{$value[0]}{$attribute[$i]} = $value[$i];
                        }
                }
        }

        return %hash;
}

################################# Active_Users #################################
# User_Name, tty, Login_Date_Time, Hostname, Idle_Time, JCPU, PCPU,            #
# Current_Process                                                              #
################################################################################
sub Active_Users {
        my %w   = get_w();
        my %who = get_who();

        foreach my $tty (keys %w) {
                my $name               = $w{$tty}{"user"};
                my $login_date_time    = $w{$tty}{"login"};
                my $login_network_name = $who{$tty}{"hostname"};
                my $idle_time          = $w{$tty}{"idle"};
                my $jcpu               = $w{$tty}{"jcpu"};
                my $pcpu               = $w{$tty}{"pcpu"};
                my $what_process       = $w{$tty}{"what"};

                print "$name,";
                print "$tty,";
                print "$login_date_time,";
                print "$login_network_name,";
                print "$idle_time,";
                print "$jcpu,";
                print "$pcpu,";
                print "$what_process\n";
        }
}

sub get_who() {
        my %hash = ();
        my $command = "/usr/bin/who -X 2>/dev/null";

        chomp(my @output = `LANG=C $command`);
        foreach my $output (@output) {
                my ($tty, $month, $day, $time, $name, $hostname) = split(/\s+/, $output);

		$hostname =~ s/\(//;
		$hostname =~ s/\)//;

                my %who = (
                        "name"     => $name,
                        "month"    => $month,
                        "day"      => $day,
                        "time"     => $time,
                        "hostname" => $hostname
                );
                $hash{$tty} = \%who;
        }

        return %hash;
}

sub get_w() {
        my %hash = ();
        my $command = "/usr/bin/w -Xh 2>/dev/null";
        chomp(my @output = `LANG=C $command`);

        foreach my $output (@output) {
                my ($tty, $login, $idle, $jcpu, $pcpu, $what, $user) = split(/\s+/, $output);

                my %w = (
                        "user"  => $user,
                        "login" => $login,
                        "idle"  => $idle,
                        "jcpu"  => $jcpu,
                        "pcpu"  => $pcpu,
                        "what"  => $what,
                );
                $hash{$tty} = \%w;
        }

        return %hash;
}

############################### Security_States ################################
# Security_Level, User_Authentication, Firewall                                #
################################################################################
sub Security_States {
        # Get lsauthent state
        my $command = "/usr/bin/lsauthent 2>/dev/null";
        chomp (my $lsauthent = `LANG=C $command`);

        # Get fireware setting
        $command="/usr/ios/cli/ioscli viosecure -firewall view | /usr/bin/grep \"^Firewall\" 2>/dev/null";
        chomp (my $output = `LANG=C $command`);
        my ($trash, $firewall) = split(/\s+/, $output);

        # Get security level
        $command = "/usr/ios/cli/ioscli viosecure -nonint -view | /usr/bin/cut -d: -f1 | /usr/bin/grep telnetd 2>/dev/null";
        chomp ($output = `LANG=C $command`);
        my ($trash, $level) = split(/telnetd/, $output);

        if ($level =~ /ls\)/ ) {
                $level = "Low";
        } elsif ($level =~ /ms\)/ ) {
                $level = "Medium";
        } elsif ($level =~ /hs\)/ ) {
                $level = "High";
        } else {
                $level = " ";
        }

        print "$level,";
        print "$lsauthent,";
        print "$firewall\n";
}

################################### Firewall ###################################
# Interface, Local_Port, Remote_Port, Service, IP_Address, Expiration_Time     #
################################################################################
sub Firewall {
	my $command = "/usr/ios/cli/ioscli viosecure -firewall view -fmt : | /usr/bin/awk 'BEGIN {RS=\"\\n\"} {if (NR>1) {print}}' 2>/dev/null";
	chomp (my @output = `LANG=C $command`);

	foreach my $line (@output) {
		my ($interface, $local_port, $remote_port, $service, $ipaddress, $expiration) = split(/:/, $line);

		if (length($expiration) == 0) {
			$expiration = " ";
		}
		
                print "$interface,";
                print "$local_port,";
                print "$remote_port,";
                print "$service,";
                print "$ipaddress,";
                print "$expiration\n";
        }
}

################################# Capabilities #################################
# Type, Status                                                                 #
################################################################################
sub Capabilities {
	my $capabilities = "disabled";
	my $environment  = get_env();
	if ($environment eq "IVM") {
		$capabilities = "enabled";
	}
	print "IVM,$capabilities\n";
}

############################### Storage_Mappings ###############################
# VIOS_Name, Hostname, IP_Address, Partition_ID, VSSA_Slot, VSSA_Name,         #
# VTD_Name, VIOS_Physical_Adapter, Disk, LV_Name, LUN_ID,                      #
# Client_Partition_Name, Client_Hostname, Client_IP_Address,                   #
# Client_Partition_ID, Client_Partition_State, VSCA_Slot                       #
################################################################################
sub Storage_Mappings {
  my $user   = "";
  my $hmc    = "";
  my $hmc_a    = "";
  my $env    = get_env();
  if ( ($env eq "IVM") || ($env eq "VIO") ) {
    my $output = "";
    my $value  = "";
    $output = `/usr/ios/cli/ioscli lssvc ITM_premium | /usr/bin/grep MANAGING_SYSTEM 2>/dev/null`;
    if ( $output =~ /^MANAGING_SYSTEM:(.*)\@(.*)/ ) {
      $hmc  = $2;
      $user = $1;
    } elsif ( $output =~ /^MANAGING_SYSTEM:(.*)/ ) {
      $hmc  = $1;
      $user = "hscroot";
    }
  }

  if ( ($env eq "VIO") || ($env eq "IVM") ) {
    if (length($hmc) <= 0) {
      chomp($hmc_a = `LANG=C /usr/bin/lsrsrc -dx IBM.ManagementServer Hostname ManagerType | /usr/bin/grep -w HMC | /usr/bin/head -1 | /usr/bin/cut -d: -f1 | /usr/bin/sed \'s/\"//g\' 2>/dev/null`);
    }

    if (length($user) <= 0) {
      $user = "hscroot";
    }
    get_vio_disk_map($env, $user, $hmc);
  }
}

#
# Print out the mapping for ITM in the following format:
# lpar_name,lpar_hostname,lpar_IP,lpar_id,VSSA_Slot,VSSA_Name,VTD,physical_adapter,disk,lv_name,lun,client_lpar_name,client_hostname,client_IP,client_lpar_id,client_lpar_state,VSCA_Slot
#
sub get_vio_disk_map {
	my ($env, $user, $hmc) = @_;
	my ($vio_id, $vio_lpar_name) = get_lslparinfo();
	my %lsdev                    = get_lsdev();
	my %lsmap                    = get_lsmap();

        my ($physical_adapter, $vssa_name, $lv_name) = (" ", " ", " ");
        my ($client_lpar_id,   $client_ip, $disk)    = (" ", " ", " ");
        my ($client_hostname,  $vsca_slot, $lun)     = (" ", " ", " ");
        my ($client_lpar_name, $vssa_slot, $vtd)     = (" ", " ", " ");
	my ($client_lpar_state)                      = (" ");
	my ($disk_tps,$disk_rps,$disk_wps) = (0, 0, 0);
	my ($vtd_tps,$vtd_rps,$vtd_wps) = (0, 0, 0);
	my ($disk_tps_prct,$disk_rps_prct, $disk_wps_prct) = (0, 0, 0);
	my ($VTD_Spans_Multiple_Disks) = (" ");
	my ($client_device_name) = (" ");

	# Get all partitions in the same managed_system as the VIO/IVM server
	my %cecs        = get_hmc_cec($env, $user, $hmc, "");
	my $vio_cec     = "";
	my %partitions  = ();
	my %lspartition = ();
        foreach my $cec (keys %cecs) {
		my $mtm_ms   = join('_', $cecs{$cec}{"type_model"}, $cecs{$cec}{"serial_num"});
                %partitions  = get_hmc_lpars($env, $cec, $user, $hmc);
		if ( exists $partitions{$vio_lpar_name} ) {
			$vio_cec     = $cec;
			%lspartition = get_lspartition($env, $mtm_ms, $user, $hmc);
			last;
		}
	}

	my %vslot     = get_squadron_vslot($env, $vio_cec, $user, $hmc);
	my %vio_vslot = ();
	if ( exists($vslot{$vio_lpar_name}) ) { %vio_vslot = %{$vslot{$vio_lpar_name}}; }

        if ($ENV{ITM_DATA_DEBUG}) {
                print "DEBUG === HMC VSLOT HASH ===\n";
                print_hash(0, %vslot);

                print "DEBUG === LSMAP ===\n";
                print_hash(0, %lsmap);
        }

	chomp(my $vio_hostname    = `hostname`);
	chomp(my $output          = `host $vio_hostname`);
	my ($trash, $trash2, $ip) = split(/\s+/, $output);
	my $vio_ip                = $ip;

	# If case of IP aliasing, host command returns alias names also. So, the vio_ip contains ',' at the end.
	# Remove ',' at the end. Defect#90510

	$vio_ip					  =~s/,//;
	if (%lsmap) {
	        # For each lsmap, look at the vhost to get the lpar_id of partition using virtual disk
        	foreach my $vhost (keys %lsmap) {
			($physical_adapter, $vssa_name, $lv_name) = (" ", " ", " ");
			($client_lpar_id,   $client_ip, $disk)    = (" ", " ", " ");
			($client_hostname,  $vsca_slot, $lun)     = (" ", " ", " ");
			($client_lpar_name, $vssa_slot, $vtd)     = (" ", " ", " ");
			($client_lpar_state)                      = (" ");

			$vtd =  $lsmap{$vhost}{"vtd"};
			if (length($vtd) < 2) {
				next;
			}

			# Get lpar_id from the vhost lsmap -all output.
                        my @physloc = split("-", $lsmap{"$vhost"}{"physloc"});
                        $vssa_slot  = $physloc[2];
                        $vssa_slot  =~ s/C//g;

			$client_lpar_id   = $vio_vslot{$vssa_slot}{"remote_lpar_id"};
			$client_lpar_name = $vio_vslot{$vssa_slot}{"remote_lpar_name"};
			$vsca_slot        = $vio_vslot{$vssa_slot}{"remote_slot_num"};
			$client_device_name = $vio_vslot{$vssa_slot}{"client_device_name"};
                       	if ( exists $lspartition{$client_lpar_id} ) {
                               	$client_hostname = $lspartition{$client_lpar_id}{"hostname"};
	                        $client_ip = $lspartition{$client_lpar_id}{"address"};
        	        }

			$vssa_name  = ($vhost =~ /:/) ? (split(":", $vhost))[0] : $vhost;
			$lun        =  $lsmap{$vhost}{"lun"};
	                my $backing =  $lsmap{$vhost}{"backing"};
			if ($lsdev{$backing}{"class"} eq "logical_volume" ) {
				$lv_name   = $backing;
	               	        my $parent = $lsdev{$lv_name}{"parent"};
				my %lspv   = get_lspv();
	                        foreach my $pv (keys %lspv) {
	                                if ($parent eq $lspv{$pv}{"vgname"}) {
        	                                $disk             = $pv;
               		                        $physical_adapter = $lsdev{$pv}{"parent"};
                       		                last;
					}
	                        }
			} else {
				$disk             = $backing;
				$physical_adapter = $lsdev{$disk}{"parent"};
			}

			$client_lpar_state = $partitions{$client_lpar_name}{"state"};

			my $iostat_command = "/usr/bin/iostat -D -l | /usr/bin/grep $disk 2>/dev/null";
		        my $iostat_output = `LANG=C $iostat_command`;
			my @iostat_output_array = split (" ", $iostat_output);
			$disk_tps = $iostat_output_array[3]; #Indicates the number of transfers per second that are issued to the disk. A transfer is of indeterminate size.
			$disk_rps = $iostat_output_array[6]; #Read operations per second
			$disk_wps = $iostat_output_array[12];#Write operations per second

			my $iostat_VTD = substr("#".$lv_name."_".$vtd,0,20);
			$iostat_command = "/usr/bin/iostat -aD -l | /usr/bin/grep \"$iostat_VTD\" 2>/dev/null";
		        $iostat_output = `LANG=C $iostat_command`;
			@iostat_output_array = split (" ", $iostat_output);
			$vtd_tps = $iostat_output_array[1]; #Indicates the number of transfers per second that are issued to the disk. A transfer is of indeterminate size.
			$vtd_rps = $iostat_output_array[4]; #Read operations per second
			$vtd_wps = $iostat_output_array[8];#Write operations per second

			if ($lv_name ne " ")
			{
				my $lslv_command = "/usr/sbin/lslv -l $lv_name | /usr/bin/wc -l 2>/dev/null";
				my $lslv_output = `LANG=C $lslv_command`;
				my $nr_pvs = 0;
				$nr_pvs = $lslv_output;
				if ( $nr_pvs == 3 ) 
				{
					$VTD_Spans_Multiple_Disks = "false";
				}
				else
				{
					$VTD_Spans_Multiple_Disks = "true";
				}

				if ($disk_tps != 0) 
				{
					$disk_tps_prct = $vtd_tps/$disk_tps*100;
				}
				if($disk_rps != 0)
				{
					$disk_rps_prct = $vtd_rps/$disk_rps*100;
				}
				if($disk_wps != 0)
				{
					$disk_wps_prct = $vtd_wps/$disk_wps*100;
				}
			}

			print "$vio_lpar_name,";
			print "$vio_hostname,";
			print "$vio_ip,";
			print "$vio_id,";
			print "$vssa_slot,";
			print "$vssa_name,";
			print "$vtd,";
			print "$physical_adapter,";
			print "$disk,";
			print "$lv_name,";
			print "$lun,";
			print "$client_lpar_name,";
			print "$client_hostname,";
			print "$client_ip,";
			print "$client_lpar_id,";
			print "$client_lpar_state,";
			print "$vsca_slot,";
			print "$vtd_tps,$vtd_rps,$vtd_wps,";
			print "$VTD_Spans_Multiple_Disks,";
			print "$disk_tps,$disk_rps,$disk_wps,";
			print "$disk_tps_prct,$disk_rps_prct,$disk_wps_prct,";
                 	print "$client_device_name\n";
		}
	} elsif ($env ne "") {
		print "$vio_lpar_name,";
		print "$vio_hostname,";
		print "$vio_ip,";
		print "$vio_id,";
		print "$vssa_slot,";
		print "$vssa_name,";
		print "$vtd,";
		print "$physical_adapter,";
		print "$disk,";
		print "$lv_name,";
		print "$lun,";
		print "$client_lpar_name,";
		print "$client_hostname,";
		print "$client_ip,";
		print "$client_lpar_id,";
		print "$client_lpar_state,";
		print "$vsca_slot,";
		print "$vtd_tps,$vtd_rps,$vtd_wps,";
		print "$VTD_Spans_Multiple_Disks,";
		print "$disk_tps,$disk_rps,$disk_wps,";
		print "$disk_tps_prct,$disk_rps_prct,$disk_wps_prct,";
               	print "$client_device_name\n";
	}
}

#
# Return a hash of all lsmap out minus networks
#
# hash => { vhost0 => { physloc => "value", 
#                       ...
#                       vtd     => "value" } }
#         { vhost1 => { ...                } }
#         { ...    => { ...                } }
#
sub get_lsmap {
        my $command = "";
        my %hash = ();

	my $delimiter = "::";
	$command = "/usr/ios/cli/ioscli lsmap -all -field SVSA Physloc \"Client Partition ID\" VTD LUN \"Backing Device\" BDPhysloc -fmt $delimiter 2>/dev/null";

	# Output will printed in the format:
	# svsa::physloc::clientid_hex::vtd::lun::backing::bdphysloc
        my @output = `LANG=C $command`;
        foreach my $output (@output) {
		chomp($output);
		my @field = split(/$delimiter/, $output);
		my $count = @field;
		my $svsa  = $field[0];
		my $clientid_hex = hex($field[2]);
		my $clientid = sprintf("%d", $clientid_hex);
		my %vhost = ();

		# Since the output for vhost can have additional mapping, where
		# vtd, lun, backing bdphysloc are repeated, a subhash will need to
		# be created.
		# For example
		#    Standard vhost mapping:
		#    svsa::physloc::clientid_hex::vtd::lun::backing::bdphysloc
		#
		#    Additional vhost mapping:
		#    svsa::physloc::clientid_hex::vtd::lun::backing::bdphysloc::vtd::lun::backing::bdphysloc::<repeats ...>
		#    <___7___>|<_4__>|<_4__>|<_4__>|...
		if ($count > 7) {
			my $lun = $field[4];
                        my %sub_vhost = (
                               "physloc"        => $field[1],
                               "clientid"       => $clientid,
                               "vtd"            => $field[3],
			       "lun"		=> $lun,
                               "backing"        => $field[5],
                               "bdphysloc"      => $field[6]
                        );
			$hash{"$svsa:$lun"} = \%sub_vhost;
	
			my $loop = ($count - 7)/4;
			for (my $i = 0; $i < $loop; $i++) {
				my $vtd       = $field[($i * 4) + 7 + 0];
				my $lun       = $field[($i * 4) + 7 + 1];
				my $backing   = $field[($i * 4) + 7 + 2];
				my $bdphysloc = $field[($i * 4) + 7 + 3];
				my %sub_vhost = (
					"physloc"        => $field[1],
					"clientid"       => $clientid,
					"vtd"            => $vtd,
					"lun"            => $lun,
					"backing"        => $backing,
					"bdphysloc"      => $bdphysloc
				);
				$hash{"$svsa:$lun"} = \%sub_vhost;
			}
		} else {
                	%vhost = (
	                       "physloc"	=> $field[1],
        	               "clientid"	=> $clientid,
                	       "vtd"		=> $field[3],
	                       "lun"		=> $field[4],
        	               "backing"	=> $field[5],
                	       "bdphysloc"	=> $field[6]
	                );
			$hash{$svsa} = \%vhost;
		}
        }

        return %hash;
}

#
# Return a hash of lspv output
# hash => { hdisk0 => { vgname => value,
#                       ...              }
#         { ...                          }
#
sub get_lspv {
        my $command = "";
        my %hash = ();

        $command = "/usr/sbin/lspv";

        my @output = `LANG=C $command`;
        foreach my $output (@output) {
                chomp($output);
                my ($name, $pv_identifier, $vgname, $state) = split(/\s+/, $output);

                my %vio = (
                        "pv_identifier" => $pv_identifier,
                        "vgname"        => $vgname,
                        "state"         => $state,
                );
                $hash{$name} = \%vio;
        }

        return %hash;
}

#
# Get predefined virtual disk
#
sub get_squadron_vslot {
        my ($env, $cec, $user, $hmc, $lpar) = @_;
        my $command = "";
        my @output = ();
        my %res;

	my ($null, $log_unit, $device_name) = ("", "", "");
	my ($lpar_name, $lpar_id, $state, $slot_num, $remote_slot_num) = ("", "", "", "", "");
	my ($adapter_type, $remote_lpar_id, $remote_lpar_name, $backing_devices, $remote_backing_device) = ("", "", "", "", "");

	if ($env eq "VIO") {
        	$command = "/usr/bin/ssh $user\@$hmc lshwres -r virtualio -m \"$cec\" --rsubtype scsi --level lpar -F lpar_name,lpar_id,state,slot_num,remote_slot_num,adapter_type,remote_lpar_id,remote_lpar_name,remote_backing_device 2>/dev/null";
	} 
	elsif ($env eq "IVM") {
		$command = "/usr/ios/lpm/bin/lshwres -r virtualio -m \"$cec\" --rsubtype scsi --level lpar -F lpar_name,lpar_id,state,slot_num,remote_slot_num,adapter_type,remote_lpar_id,remote_lpar_name 2>/dev/null";
	}

        @output = `LANG=C $command`;
        if ( $? eq "0" ) {
                my $count = @output;
                for (my $i = 0; $i < $count; $i++) {
                        if  ( ($output[$i] ne "") && ($output[$i] !~ /HSCL/) && ($output[$i] !~ /No results were found/) ) {
                        	chomp($output[$i]);
				($lpar_name, $lpar_id, $state, $slot_num, $remote_slot_num, $adapter_type, $remote_lpar_id, $remote_lpar_name,$remote_backing_device) = split(/,/, $output[$i]);
#				my @backing = split("/", $backing_devices);
#				if (@backing == 3) {
#					$device_name = $backing[2];
#				} else {
#					$device_name = "";
#				}
				my @output_array = split(/"/, $output[$i]);
				if (@output_array > 1) {
					my @remote_backing_device_array = split(",",$output_array[1]);
					$remote_backing_device = join("|",@remote_backing_device_array);
				}

				$res{$lpar_name}{$slot_num}{"device_name"}      = $device_name;
				$res{$lpar_name}{$slot_num}{"lpar_id"}          = $lpar_id;
				$res{$lpar_name}{$slot_num}{"remote_lpar_id"}   = $remote_lpar_id;
				$res{$lpar_name}{$slot_num}{"remote_lpar_name"} = $remote_lpar_name;
				$res{$lpar_name}{$slot_num}{"remote_slot_num"}  = $remote_slot_num;
				$res{$lpar_name}{$slot_num}{"state"}            = $state;
				$res{$lpar_name}{$slot_num}{"client_device_name"} = $remote_backing_device;
                        }
                }
        }

        return %res;
}

############################### Network_Mappings ###############################
# VLAN_ID, Partition_Name, Partition_State, Hostname, IP_Address,              #
# Partition_ID, VEA_Slot, VEA_MAC, VEA_IP_address, Trunk,                      #
# Shared_Ethernet_Adapter, SEA_IP_Address, SEA_MAC,                            #
# Physical_Ethernet_Adapters, Virtual_Ethernet_Adapters, Failover, Priority,   #
# Bridging, Control_Channel                                                    #
################################################################################
sub Network_Mappings {
  my $user   = "";
  my $hmc    = "";
  my $hmc_a    = "";
  my $env    = get_env();
  if ( ($env eq "IVM") || ($env eq "VIO") ) {
    my $output = "";
    my $value  = "";
    $output = `/usr/ios/cli/ioscli lssvc ITM_premium | /usr/bin/grep MANAGING_SYSTEM 2>/dev/null`;
    if ( $output =~ /^MANAGING_SYSTEM:(.*)\@(.*)/ ) {
      $hmc  = $2;
      $user = $1;
    } elsif ( $output =~ /^MANAGING_SYSTEM:(.*)/ ) {
      $hmc  = $1;
      $user = "hscroot";
    }
  }

  if ( ($env eq "VIO") || ($env eq "IVM") ) {
    if (length($hmc) <= 0) {
      chomp($hmc_a = `LANG=C /usr/bin/lsrsrc -dx IBM.ManagementServer Hostname ManagerType | /usr/bin/grep -w HMC | /usr/bin/head -1 | /usr/bin/cut -d: -f1 | /usr/bin/sed \'s/\"//g\' 2>/dev/null`);
    }

    if (length($user) <= 0) {
      $user = "hscroot";
    }
    get_vio_net_map($env, $user, $hmc);
  }
}

#
# Print out the mapping in the following format:
# vlan_id,lpar_name,state,client_hostname,client_IP,lpar_id,VEA_Slot,VEA_MAC,VEA_IP,trunk,SEA,SEA_IP,SEA_MAC,physical_ethernet,virtual_adapter,failover,priority,bridging,control_channel
#
sub get_vio_net_map {
        my ($env, $user, $hmc) = @_;
        my ($vio_id, $vio_lpar_name) = get_lslparinfo();
        my %lsdev = get_lsdev();
        my %lsmap_net = get_lsmap_net();

        my ($client_lpar_id,    $bridging,  $vea_mac,  $trunk) = (" ", " ", " ");
        my ($physical_ethernet, $vio_lanid, $failover, $sea)   = (" ", " ", " ");
        my ($client_hostname,   $client_ip, $vea_ip)           = (" ", " ", " ");
        my ($client_lpar_name,  $priority,  $sea_mac)          = (" ", " ", " ");
        my ($control_channel,   $vea_slot,  $sea_ip)           = (" ", " ", " ");
        my ($virtual_adapter,   $trunk,     $sea)              = (" ", " ", " ");
	my ($client_lpar_state)                                = (" ");

        # Get all partitions in the same managed_system as the VIO/IVM server
	my %cecs        = get_hmc_cec($env, $user, $hmc, "");
	my $vio_cec     = "";
	my %partitions  = ();
	my %lspartition = ();
        foreach my $cec (keys %cecs) {
		my $mtm_ms   = join('_', $cecs{$cec}{"type_model"}, $cecs{$cec}{"serial_num"});
                %partitions  = get_hmc_lpars($env, $cec, $user, $hmc);
                if ( exists $partitions{$vio_lpar_name} ) {
                        $vio_cec     = $cec;
			%lspartition = get_lspartition($env, $mtm_ms, $user, $hmc);
                        last;
                }
        }
        my %veth = get_squadron_veth($env, $vio_cec, $user, $hmc);

	# Get VIO Hostname and VIO IP address
        chomp(my $vio_hostname    = `hostname`);
        chomp(my $output          = `host $vio_hostname`);
        my ($trash, $trash2, $ip) = split(/\s+/, $output);
        my $vio_ip                = $ip;

	# If case of IP aliasing, host command returns alias names also. So, the vio_ip contains ',' at the end.
	# Remove ',' at the end. Defect#90510

	$vio_ip					  =~s/,//;
	# Get all VIO vlan ids
	my @vio_vlan_id;
	foreach my $mac (keys %{$veth{$vio_lpar_name}}) {
		push(@vio_vlan_id, $veth{$vio_lpar_name}{$mac}{"port_vlan_id"});
		if ( exists($veth{$vio_lpar_name}{$mac}{"addl_vlan_ids"}) ) {
			push(@vio_vlan_id, split(/,/, $veth{$vio_lpar_name}{$mac}{"addl_vlan_ids"}));
		}
	}

	# Get the VIO Virtual Ethernet hash so we can track which SEA, Physical ethernet, and SEA MAC.	
	my %vio_veth_hash = %{$veth{$vio_lpar_name}};

	if ($ENV{ITM_DATA_DEBUG}) {
		print "DEBUG === HMC VIO_VETH_HASH ===\n";
                print_hash(0, %lsmap_net);

		print "DEBUG === HMC VIO_VETH_HASH ===\n"; 
		print_hash(0, %vio_veth_hash);

		print "DEBUG === VETH ===\n"; 
		print_hash(0, %veth);
	}

	if (%veth) {
	        # For each veth, loop through and map the vlans
        	foreach my $lpar (keys %veth) {
			($client_lpar_id,    $bridging,  $vea_mac,  $trunk) = (" ", " ", " ");
			($physical_ethernet, $vio_lanid, $failover, $sea)   = (" ", " ", " ");
			($client_hostname,   $client_ip, $vea_ip)           = (" ", " ", " ");
			($client_lpar_name,  $priority,  $sea_mac)          = (" ", " ", " ");
			($control_channel,   $vea_slot,  $sea_ip)           = (" ", " ", " ");
			($virtual_adapter,   $trunk,     $sea)              = (" ", " ", " ");
			($client_lpar_state)                                = (" ");

			my %hash = %{$veth{$lpar}};
			foreach my $mac_address (keys %hash) {
				my $client_vlan_id = $hash{$mac_address}{"port_vlan_id"};
				if ($lpar eq $vio_lpar_name) {
					foreach my $veth_ent (keys %lsmap_net) {
						my $physloc = $lsmap_net{$veth_ent}{"physloc"};
						my ($trash1, $trash2, $lsmap_net_client_slot) = split("-", $physloc);
						$lsmap_net_client_slot =~ s/C//;

						# Match on the SEA slot on the VIO server
						if ($hash{$mac_address}{"slot_num"} eq $lsmap_net_client_slot) {
							$priority          =  $hash{$mac_address}{"trunk_priority"};
							$virtual_adapter   = ($veth_ent =~ /:/) ? (split(":", $veth_ent))[0] : $veth_ent;
							if ($lsdev{$virtual_adapter}{"status"} =~ /Available/) {
								my $virt_adapt_en  =  $virtual_adapter;
								$virt_adapt_en     =~ s/t//g;
								my %lsattr         =  get_lsattr_ent($virt_adapt_en);
								$vea_ip            =  $lsattr{"netaddr"};
								if ($ENV{ITM_DATA_DEBUG}) {
									print "DEBUG === LSATTR HASH FOR $virt_adapt_en ===\n";
									print_hash(0, %lsattr);
								}

								if ($priority eq "unavailable") {
									my %virt_hash =  get_sea_mac($virt_adapt_en);
									$priority     = (exists($virt_hash{"priority"})) ? $virt_hash{"priority"} : $priority; 
	                                                                if ($ENV{ITM_DATA_DEBUG}) {
        	                                                                print "DEBUG === LSATTR HASH FOR $virt_adapt_en ===\n";
                	                                                        print_hash(0, %virt_hash);
                        	                                        }
								}
							} else {
								$virtual_adapter = " ";
								$client_vlan_id  = " ";
								next;
							}

							$sea               =  $lsmap_net{$veth_ent}{"sea"};
							if ($lsdev{$sea}{"status"} =~ /Available/) {
								my %sea_hash       =  get_sea_mac($sea);
								$sea_mac           =  $sea_hash{$sea};
                	                                        $bridging          =  ($sea_hash{"bridging"} eq "True") ? "yes" : "no";
                        	                                if ($ENV{ITM_DATA_DEBUG}) {
									print "DEBUG === ENTSTAT HASH FOR $sea ===\n";
	                                                                print_hash(0, %sea_hash);
	                                                        }

								my %lsattr 	   =  get_lsattr_ent($sea);
								$control_channel   =  ($lsattr{"ctl_chan"} ne "") ? $lsattr{"ctl_chan"} : " ";
								$failover          =  $lsattr{"ha_mode"};
								$physical_ethernet =  $lsattr{"real_adapter"};
                                                                if ($ENV{ITM_DATA_DEBUG}) {
                                                                        print "DEBUG === LSATTR HASH FOR $sea ===\n";
                                                                        print_hash(0, %lsattr);
                                                                }
								if ($lsdev{$physical_ethernet}{"description"} =~ /EtherChannel/) {
									my $adapters = ();
									my %lsattr      =  get_lsattr($physical_ethernet);
	
									if ( exists($lsattr{"adapter_names"}) ) {
										$adapters = $lsattr{"adapter_names"};
										$adapters =~ s/,/ /g;
									}
									if ( exists($lsattr{"backup_adapter"}) && ($lsattr{"backup_adapter"} ne "NONE") ) {
										my $backup = $lsattr{"backup_adapter"};
										$adapters = "$adapters $backup";
									}
									$physical_ethernet = "$physical_ethernet:$adapters";
        	                                                        if ($ENV{ITM_DATA_DEBUG}) {
                	                                                        print "DEBUG === LSATTR HASH FOR $physical_ethernet ===\n";
                        	                                                print_hash(0, %lsattr);
                                	                                }
								}

								my $sea_adapter    =  $sea;
								$sea_adapter       =~ s/t//g;
								%lsattr            =  get_lsattr($sea_adapter);
								$sea_ip            =  $lsattr{"netaddr"};
								if ($ENV{ITM_DATA_DEBUG}) {
									print "DEBUG === LSATTR HASH FOR $sea_adapter ===\n";
									print_hash(0, %lsattr);
								}
							} else {
								# Blank out this value related to SEA if sea is in Defined state
					                        ($physical_ethernet, $failover, $sea_mac, $sea) = (" ", " ", " ", " ");
					                        ($control_channel,   $bridging, $sea_ip)        = (" ", " ", " ");
							}

							# if additional vlan id is populated, then we need to create a new table listing
							if ( exists($hash{$mac_address}{"addl_vlan_ids"}) ) {
								my @addl_vlan_ids  = split(/,/, $hash{$mac_address}{"addl_vlan_ids"});
			                                        $client_lpar_name  = $lpar;
                        			                $client_lpar_id    = $partitions{$client_lpar_name}{"lpar_id"};
			                                        $client_lpar_state = $partitions{$client_lpar_name}{"state"};

			                                        if ( exists $lspartition{$client_lpar_id} ) {
                        			                        $client_hostname = $lspartition{$client_lpar_id}{"hostname"};
                                               				$client_ip       = $lspartition{$client_lpar_id}{"address"};
			                                        }

		        	                                $vea_mac   = $mac_address;
                	        		                $vio_lanid = $hash{$mac_address}{"port_vlan_id"};
								$trunk     = $hash{$mac_address}{"is_trunk"};
								$vea_slot  = $hash{$mac_address}{"slot_num"};
								foreach my $vlan (@addl_vlan_ids) {
									print "$vlan,";
									print "$client_lpar_name,";
									print "$client_lpar_state,";
									print "$client_hostname,";
									print "$client_ip,";
									print "$client_lpar_id,";
									print "$vea_slot,";
									print "$vea_mac,";
									print "$vea_ip,";
									print "$trunk,";
									print "$sea,";
									print "$sea_ip,";
									print "$sea_mac,";
									print "$physical_ethernet,";
									print "$virtual_adapter,";
									print "$failover,";
									print "$priority,";
									print "$bridging,";
									print "$control_channel\n";
								}
							}

							last;
						}
					}
				}

				if (exists_array($client_vlan_id, @vio_vlan_id)) {
			                $client_lpar_name  = $lpar;
			                $client_lpar_id    = $partitions{$client_lpar_name}{"lpar_id"};
					$client_lpar_state = $partitions{$client_lpar_name}{"state"};

			                if ( exists $lspartition{$client_lpar_id} ) {
			                        $client_hostname = $lspartition{$client_lpar_id}{"hostname"};
		        	                $client_ip       = $lspartition{$client_lpar_id}{"address"};
	        	        	}

		                        $vea_mac   = $mac_address;
       			                $vio_lanid = $hash{$mac_address}{"port_vlan_id"};
               			        $trunk     = $hash{$mac_address}{"is_trunk"};
                       			$vea_slot  = $hash{$mac_address}{"slot_num"};

		                        print "$vio_lanid,";
       			                print "$client_lpar_name,";
					print "$client_lpar_state,";
					print "$client_hostname,";
					print "$client_ip,";
					print "$client_lpar_id,";
		                        print "$vea_slot,";
       			                print "$vea_mac,";
               			        print "$vea_ip,";
                       			print "$trunk,";
					print "$sea,";
		                        print "$sea_ip,";
       			                print "$sea_mac,";
               			        print "$physical_ethernet,";
					print "$virtual_adapter,";
       	                	 	print "$failover,";
					print "$priority,";
					print "$bridging,";
					print "$control_channel\n";
				}
			}
        	} 
	} else {
                print "$vio_lanid,";
                print "$client_lpar_name,";
		print "$client_lpar_state,";
                print "$client_hostname,";
                print "$client_ip,";
                print "$client_lpar_id,";
                print "$vea_slot,";
                print "$vea_mac,";
                print "$vea_ip,";
                print "$trunk,";
                print "$sea,";
                print "$sea_ip,";
                print "$sea_mac,";
                print "$physical_ethernet,";
                print "$virtual_adapter,";
                print "$failover,";
                print "$priority,";
                print "$bridging,";
                print "$control_channel\n";
	}
}

#
# Return a hash of all lsmap networks
#
# hash => { svea1 => { physloc => "value",
#                      ...
#                      sea     => "value" } }
#         { svea2 => { ...                } }
#         { ...   => { ...                } }
#
sub get_lsmap_net {
        my $command = "";
        my %hash = ();
        my $delimiter = "::";

        # Output will printed in the format:
        # svsa::physloc::sea::backing::bdphysloc
        $command = "/usr/ios/cli/ioscli lsmap -all -net -field svea physloc sea backing bdphysloc -fmt $delimiter 2>/dev/null";

        # Output will printed in the format:
        # svsa::physloc::sea::backing::bdphysloc
        my @output = `LANG=C $command`;
        foreach my $output (@output) {
                chomp($output);
                my @field   = split(/$delimiter/, $output);
                my $count   = @field;
                my $svsa    = $field[0];
				my $physloc = $field[1];
                my %vhost = ();

                # Since the output for vhost can have additional mapping, where
                # vtd, lun, backing bdphysloc are repeated, a subhash will need to
                # be created.
                # For example
                #    Standard vhost mapping:
                #    svsa::physloc::sea::backing::bdphysloc
                #
                #    Additional vhost mapping:
                #    svsa::physloc::sea::backing::bdphysloc::sea::backing::bdphysloc::<repeats ...>
                #    <__5__>|<_3_>|<_3_>|<_3_>|...
                if ($count > 5) {
                        my $sea = $field[2];
                        my %sub_vhost = (
				"physloc"        => $physloc,
				"sea"            => $sea,
				"backing"        => $field[3],
				"bdphysloc"      => $field[4]
                        );
                        $hash{"$svsa:$sea"} = \%sub_vhost;

                        my $loop = ($count - 5)/3;
                        for (my $i = 0; $i < $loop; $i++) {
                                my $sea       = $field[($i * 3) + 5 + 0];
                                my $backing   = $field[($i * 3) + 5 + 1];
                                my $bdphysloc = $field[($i * 3) + 5 + 2];
                                my %sub_vhost = (
                                        "physloc"        => $physloc,
					"sea"		 => $sea,
                                        "backing"        => $backing,
                                        "bdphysloc"      => $bdphysloc
                                );
                                $hash{"$svsa:$sea"} = \%sub_vhost;
                        }
                } else {
                        %vhost = (
				"physloc"        => $physloc,
				"sea"		 => $field[2],
				"backing"        => $field[3],
				"bdphysloc"      => $field[4]
                        );
                        $hash{$svsa} = \%vhost;
                }
        }

        return %hash;
}

#
# Get SEA macaddress 
# Return a hash containing the SEA macaddress
# The hash may contain the following information:
# hash => { en3 => 00:09:6B:DD:43:B2,
#           en0 => 00:09:6B:DD:43:B2,
#           en2 => F2:D4:62:5A:E4:E3 }
#
sub get_sea_mac {
        my ($adapter) = @_;
	my %hash = ();

	my $command = "/usr/bin/entstat -d $adapter 2>/dev/null";
	my @output = `LANG=C $command`;

	chomp(@output);
	my $count = @output;
	for (my $i = 0; $i < $count; $i++) {
		if ( ($output[$i] ne "") && ($output[$i] =~ /Priority: /) ) {
                        my ($unuse1, $unuse2, $priority) = split(/\s+/, $output[$i]);
                        $hash{"priority"} = $priority;
                }
                if ( ($output[$i] ne "") && ($output[$i] =~ /Active: /) ) {
                        my ($unuse1, $unuse2, $unuse3, $unuse4, $bridging) = split(/\s+/, $output[$i]);
                        $hash{"bridging"} = $bridging;
                }

		if ( ($output[$i] ne "") && ($output[$i] =~ /ETHERNET STATISTICS/) ) {

			# String Example:
			#    ETHERNET STATISTICS (ent3) :
			my ($unuse1, $unuse2, $net_adapter) = split(/\s+/, $output[$i]);
			$net_adapter =~ s/^\(|\)$//g;

			until ($output[$i] =~ /Hardware Address/) {
				$i++;
			}

			# String Example:
			#    Hardware Address: 00:09:6b:dd:43:b2
			my ($unuse3, $mac) = split(": ", $output[$i]);
			$mac =~ s/://g;
		
			$hash{$net_adapter} = uc($mac);
		} 
	}

	return %hash;
}

#
# Determine if a value exist in an array
#
sub exists_array {
        my ($value, @array) = @_;
        foreach my $array_value (@array) {
		if ($value eq $array_value) {
			return 1;
		}
	}

	return 0;
}

#
# Get virtual ethernet mac address
#
sub get_squadron_veth {
        my ($env, $cec, $user, $hmc, $lpar) = @_;
	my $command = "";
        my @output = ();
        my %res;

	if ($env eq "VIO") {
		$command = "/usr/bin/ssh $user\@$hmc lshwres -r virtualio -m \"$cec\" --rsubtype eth --level lpar -F lpar_name,lpar_id,slot_num,state,is_required,is_trunk,ieee_virtual_eth,port_vlan_id,mac_addr,trunk_priority,addl_vlan_ids 2>/dev/null";
	} elsif ($env eq "IVM") {
		$command = "/usr/ios/lpm/bin/lshwres -r virtualio -m \"$cec\" --rsubtype eth --level lpar -F lpar_name,lpar_id,slot_num,state,is_required,is_trunk,ieee_virtual_eth,port_vlan_id,mac_addr,addl_vlan_ids 2>/dev/null";
	}

        @output = `LANG=C $command`;
        if ($? eq "0") {
                my $count = @output;
                for (my $i = 0; $i < $count; $i++) {
                        if ( ($output[$i] ne "") && ($output[$i] !~ /HSCL/) && ($output[$i] !~ /No results were found/) ) {
				my ($lpar_name, $lpar_id, $slot_num, $state, $is_required) = undef;
				my ($is_trunk,  $ieee_virtual_eth,   $port_vlan_id)        = undef;
				my ($mac_addr,  $trunk_priority,     @addl_vlan_ids)       = undef;

                                chomp($output[$i]);
				if ($env eq "VIO") {
					($lpar_name, $lpar_id, $slot_num, $state, $is_required, $is_trunk, $ieee_virtual_eth, $port_vlan_id, $mac_addr, $trunk_priority, @addl_vlan_ids) = split(/,/, $output[$i]);
				} elsif ($env eq "IVM") {
					($lpar_name, $lpar_id, $slot_num, $state, $is_required, $is_trunk, $ieee_virtual_eth, $port_vlan_id, $mac_addr, @addl_vlan_ids) = split(/,/, $output[$i]);
				}

				if ($is_trunk eq "1") { 
					$is_trunk = "yes";
				} else {
					$is_trunk = "";
				}

				if ($state eq "1") {
					$state = "yes";
				} else {
					$state = "";
				}

				my $addl_vlan_ids = join(",", @addl_vlan_ids);
				$addl_vlan_ids =~ s/"//g;
				$res{$lpar_name}{$mac_addr}{"addl_vlan_ids"}    = $addl_vlan_ids;
				$res{$lpar_name}{$mac_addr}{"ieee_virtual_eth"} = $ieee_virtual_eth;
				$res{$lpar_name}{$mac_addr}{"is_required"}      = $is_required;
				$res{$lpar_name}{$mac_addr}{"is_trunk"}         = $is_trunk;
				$res{$lpar_name}{$mac_addr}{"lpar_id"}	        = $lpar_id;
				$res{$lpar_name}{$mac_addr}{"port_vlan_id"}     = $port_vlan_id;
				$res{$lpar_name}{$mac_addr}{"slot_num"}         = $slot_num;
				$res{$lpar_name}{$mac_addr}{"state"}            = $state;

				if ($env eq "VIO") {
					$res{$lpar_name}{$mac_addr}{"trunk_priority"} = $trunk_priority;
				}
                        }
                }
        }

        return %res;
}

#
# Return a hash of lsattr ethernet information
#
sub get_lsattr_ent {
	my ($adapter) = @_;
	my %hash = ();

        my $command = "/usr/sbin/lsattr -F \"attribute:value\" -l $adapter 2>/dev/null";
        my @output = `LANG=C $command`;

	# The 'netaddr' are not valid for sea adapters
	$hash{"name"} = $adapter;
	if (@output > 1) {
	        foreach my $output (@output) {
        	        chomp($output);
                	my ($attribute, $value) = split(/:/, $output);
			if ($attribute eq "ctl_chan"     ) { $hash{"ctl_chan"}      = $value; }
                        if ($attribute eq "ha_mode"      ) { $hash{"ha_mode"}       = $value; }
                        if ($attribute eq "pvid_adapter" ) { $hash{"pvid_adapter"}  = $value; }
                        if ($attribute eq "real_adapter" ) { $hash{"real_adapter"}  = $value; }
                        if ($attribute eq "virt_adapters") { $hash{"virt_adapters"} = $value; }
                        if ($attribute eq "netaddr"      ) { $hash{"netaddr"}       = $value; }
		}
	}

        return %hash;
}

######################### Additional Helper Functions ##########################

# 
# Return a hash of lsdev output
# The hash will contain the fillowing:
# hash => { en0         => { type  => value,
#                            class => value,
#                            ...             } }
#         { device_name => { type => value,
#                            class => value,
#                            ...             } }
#         { ...                              } }          
#
sub get_lsdev {
        my $command = "";
        my %hash = ();

        my $delimiter = ":";
        $command = "/usr/sbin/lsdev -C -F 'name:status:type:class:subclass:location:physloc:parent:ddins:description' 2>/dev/null";

        my @output = `LANG=C $command`;
        foreach my $output (@output) {
                chomp($output);
                my ($name, $status, $type, $class, $subclass, $location, $physloc, $parent, $ddins, $description) = split(/$delimiter/, $output);

                my %vio = (
			"status"	=> $status, 
			"type"	 	=> $type, 
			"class"	 	=> $class,
			"subclass"	=> $subclass, 
			"location" 	=> $location, 
			"physloc"	=> $physloc, 
			"parent" 	=> $parent,
			"ddins"		=> $ddins, 
			"description"	=> $description
                );
                $hash{$name} = \%vio;
        }

        return %hash;
}

#
# Determine the environment of the running machine.
# Return string of "VIO" or "IVM"
#
sub get_env {
	my $command = "/usr/ios/lpm/bin/lsivm 2>/dev/null";
	my $env = "";

	my $output = `LANG=C $command 2>/dev/null`;
	if ($output ne "") {
		$env = "IVM";
	} elsif ( (-e "/usr/ios/cli/ioscli") && ($output eq "") ) {
		$env = "VIO";
	}

        return $env;
}

#
# Get HMC information, such as user and the HMC name
#
sub get_hmc_info {
        my ($hmc) = @_;
        my $command = "";

        my ($user, $hmc_server) = split(/@/, $hmc);
        if ($hmc_server) {
                $hmc = $hmc_server;
        } else {
                $user = "hscroot";
        }

        return ($user, $hmc);
}

#
# Return an array containing the partition information
# for the VIO server.
# The array return contains the following content:
# @array = (lpar_id, lpar_name);
#
sub get_lslparinfo {
	my $command = "/usr/ios/cli/ioscli lslparinfo 2>/dev/null";
	my @array = ();

	my $output = `LANG=C $command`;
	chomp($output);
        my ($lpar_id, @lpar_name_array) = split(" ", $output);
        my $lpar_name = join(" ",@lpar_name_array);
	push(@array, $lpar_id);
	push(@array, $lpar_name);

	return @array;
}

#
# This subroutine prints out the hash output
# Use for debugging, counts is the number of space
# to initially start.  Use 0.
#
sub print_hash {
	my ($count, %hash) = @_;

	my $string = "";
	for (my $i = 0; $i < $count; $i++) {
		$string = $string . " ";
	}
	$count+= 3;

        foreach my $category (sort (keys %hash)) {
		if (ref(\$hash{$category}) eq "SCALAR") {
			print "$string$category = $hash{$category}\n";
		}
		else {
	               	print "$string$category => {\n";
			print_hash($count, %{$hash{$category}});
        	        print "$string}\n";
		}
        }
}

#              LPAR PORTION              #
##########################################

#
# Return a hash of all managed system for the hmc.
#
# hash => { cec name => { "name   => "cec_name1,
#                         "state" => "cec_state }
#         { cec name => { ...                   }
#         { ...      => { ...                   } }
#
sub get_hmc_cec {
        my ($env, $user, $hmc, $managed_system) = @_;
	my %hash = ();
        my $command = "";

	my $split_value = ",";
	if ($env eq "VIO" ) {
	        if ($managed_system eq "") {
        	        $command = "/usr/bin/ssh $user\@$hmc lssyscfg -r sys -F name,type_model,serial_num 2>/dev/null";
        	} else {
        	        $command = "/usr/bin/ssh $user\@$hmc lssyscfg -r sys -m \"$managed_system\" -F name,type_model,serial_num 2>/dev/null";
	        }
	} else {
		if ($managed_system eq "") {
                        $command = "/usr/ios/lpm/bin/lssyscfg -r sys -F name,type_model,serial_num 2>/dev/null";
                } else {
                        $command = "/usr/ios/lpm/bin/lssyscfg -r sys -m \"$managed_system\" -F name,type_model,serial_num 2>/dev/null";
                }
	}

	
        chomp(my @output = `LANG=C $command`);
	foreach my $output (@output) {
                my ($name, $type_model, $serial_num) = split (/$split_value/, $output);
                $hash{$name}{"type_model"} = $type_model;
                $hash{$name}{"serial_num"} = $serial_num;
        }

        return %hash;
}

#
# Input : $cec = the name of the cec
#
# Return : A hash of lpar information
#
# hash => { lpar_name1 => { "default_profile" => $default_profile } }
#         { lpar_name2 => { "default_profile" => $default_profile } }
#         { ...           {                                       } }
#
sub get_hmc_lpars {
        my ($env, $cec, $user, $hmc) = @_;
        my %hash = ();
        my $command = "";

	my $split_value = ",";
	if ($env eq "VIO" ) {
	        $command = "/usr/bin/ssh $user\@$hmc lssyscfg -r lpar -m \"$cec\" -F name,lpar_id,state 2>/dev/null";
	} else {
		$command = "/usr/ios/lpm/bin/lssyscfg -r lpar -m \"$cec\" -F name,lpar_id,state 2>/dev/null";
	}

        chomp(my @output = `LANG=C $command`);
	foreach my $output (@output) {
		my ($name, $lpar_id, $state) = split (/$split_value/, $output);
		$hash{$name}{"lpar_id"} = $lpar_id;
		$hash{$name}{"state"}   = $state;
        }

        return %hash;
}

#
# Return a hash of partition ipaddress and hostname
#
sub get_lspartition {
        my ($env, $mtm_ms, $user, $hmc) = @_;
        my $command = "";

        my %hash;
	if ($env eq "VIO") {
		$command = "/usr/bin/ssh $user\@$hmc lspartition -c $mtm_ms -ix 2>/dev/null";
        	my @output = `LANG=C $command`;
	        foreach my $output (@output) {
        	        chomp($output);
			my @partition = split(/;/, $output);
			foreach my $partition (@partition) {
				my ($lpar_id, $address, $active, $hostname, $OStype, $OSlevel) = split(/:/, $partition);
	        	        my $name = $output;
				$hash{$lpar_id}{"address"}  = $address;
				$hash{$lpar_id}{"active"}   = $active;
				$hash{$lpar_id}{"hostname"} = $hostname;
				$hash{$lpar_id}{"OStype"}   = $OStype;
				$hash{$lpar_id}{"OSlevel"}  = $OSlevel;
			} 
        	}
	}

        return %hash;
}

sub WPAR_Filesystem {
  my $command = "/usr/sbin/lswpar -cM -d,";
  my $x = 0;
  foreach my $line (`$command`) {
	if ( $x ne 0) {
		print $line;
	}
	$x = 1;
  }
}

sub WPAR_CPU {
  my $lpar = `/usr/bin/iostat`;
  $lpar =~ m/tty:.*\n/;
  $' =~ m/^\s*\S+\s*\S+\s*\S+\s*\S+\s*\S+\s*\S+\s*\S+\s*(\S+)/;
  my $entc = $1;

  foreach my $line (`/usr/sbin/lswpar -s A -qca name,cpu`) {
    $line =~ m/([^:]+):([^:]+)/;
    my $wpar = $1;
    my $limits = $2;
    $limits =~ m/\d+%-(\d+%),(\d+%)/;
    my $soft = $1;
    my $hard = $2;
    $hard =~ m/(\d+)%/;
    my $hard_num = $1;
    my $out = `/usr/bin/iostat -@ $wpar`;
    $out =~ m/lcpu=(\S+)/;
    my $lcpu = $1;
    $out =~ m/ent=(\S+)/;
    my $ent = $1;
    $out =~ m/tty:.*\n/;
    $' =~ m/^\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/;
    my $lparpct = ($7/$ent);
    print $wpar . "," . $3 . "," . $4 . "," . $7 . "," . $8 . "," . $lparpct . "," . ($entc * $hard_num / 100) . "," . $hard . "," . $ent . "\n"; 
  }
}

sub WPAR_Information { 
  my $command = "/usr/sbin/lswpar -qca name";

  foreach my $name (`$command`) {
    # this gets us each wpar by name
    my $part1 = `/usr/sbin/lswpar -d, -qca name,type,state,owner,hostname,directory,auto,privateusr,checkpointable,application,active,rset,shares_cpu $name`;
    my $part2 = `/usr/sbin/lswpar -qca cpu $name`;
    my $part3 = `/usr/sbin/lswpar -d, -qca shares_memory $name`;
    my $part4 = `/usr/sbin/lswpar -qca memory $name`;
    my $part5 = `/usr/sbin/lswpar -d, -qca procvirtmem,totalprocesses,totalthreads,opname,oppid,opstart $name`;
#    my $part6 = `/usr/sbin/lswpar -Nqca interface $name 2>/dev/null`;
    $part2 =~ s/-/,/g;
    $part4 =~ s/-/,/g;
    $part2 =~ s/%//g;
    $part4 =~ s/%//g;

    $part1 =~ s/\n/,/;
    $part2 =~ s/\n/,/;
    $part3 =~ s/\n/,/;
    $part4 =~ s/\n/,/;
    $part5 =~ s/\n/,/;
    if ($part2 eq ",") {
	$part2 = ",,,";
    } 
    if ($part4 eq ",") {
	$part4 = ",,,";
    }
    
    # convert to enum format
    # 0 - Application WPAR
    # 1 - System WPAR
    $part1 =~ s/^([^,]+),A/$1,0/;
    $part1 =~ s/^([^,]+),S/$1,1/;

    # 0 - Defined, 1 - Loaded, 2 - Active, 3 - Frozen,
    # 4 - Paused, 5 - Transitional, 6 - Broken
    $part1 =~ s/^([^,]+),([^,]+),D/$1,$2,0/;
    $part1 =~ s/^([^,]+),([^,]+),L/$1,$2,1/;
    $part1 =~ s/^([^,]+),([^,]+),A/$1,$2,2/;
    $part1 =~ s/^([^,]+),([^,]+),F/$1,$2,3/;
    $part1 =~ s/^([^,]+),([^,]+),P/$1,$2,4/;
    $part1 =~ s/^([^,]+),([^,]+),T/$1,$2,5/;
    $part1 =~ s/^([^,]+),([^,]+),B/$1,$2,6/;

#    print $part1 . $part2 ."\n";
    my $all = $part1 . $part2 . $part3 . $part4 . $part5 . "\n";
    while ($all =~s/,no,/,0,/) {
    }
    while ($all =~s/,yes,/,1,/) {
    }
    print $all;
  }
}
sub WPAR_Network {
  my $command = "/usr/sbin/lswpar -d, -Nqa name,interface,address,netmask,broadcast 2>&1";
  foreach my $line (`$command`) {
    if ($line =~ m/has no network configuration/) {
    } 
    else {
      print $line;	
    } 
  }
}
sub WPAR_Physical_Memory {
  my $out = `/usr/bin/svmon`;
  #$out =~ m/memory\s*(\d+)/;
  $out =~ m/memory\s*(\d+)\s*(\d+)\s*(\d+)/; 
  my $sysmem = ($1/256);
  my $out2 = ($2/256);
  my $out3 = ($3/256);
  
  foreach my $line (`/usr/sbin/lswpar -qca name,memory`) {
    $line =~ m/([^:]+):([^:]+)/;
    my $wpar = $1;
    my $limits = $2;
    $limits =~ s/\n//;
    $limits =~ m/\d+%-(\d+%),(\d+%)/;
    my $hardcap = $2;
    my $svout = `/usr/bin/svmon -@ $wpar 2>/dev/null`;
    
    my $sv1;
    my $sv2;
    my $sv3;
    if ($svout =~ m/memory\s*(\d+)\s*(\d+)\s*(\d+)/ ) {
      $sv1 = ($1/256);
      $sv2 = ($2/256);
      $sv3 = ($3/256);
    }
    else {
      if ($svout =~ m/memory\s*-\s*(\d+)\s*-/) {
	$sv1 = $sysmem;
	$sv2 = ($1/256);
	$sv3 = $out3;
      }
    }
 
    $wpar =~ s/\n//;
    if ($sv2 ne "") {
      print $wpar . "," . $sv1 . "," . $sv2 . "," . $sv3 . "," . 100*($sv3/($sv2+$sv3)) . "," . 100*($sv2/($sv2+$sv3)) . "," . 100*($sv2/$sysmem) . "," . $hardcap . "," . $sysmem . "\n";
    }
  }
}

sub MPIO_Statistics {
  my $outz = `/usr/sbin/lspath -F name,parent,path_status,status,connection`;
  print $outz;
}

sub MPIO_Attributes {
  foreach my $dev (`/usr/sbin/lspath -F name`) {
    $dev =~ m/(.*)\n/;
    my $outz = `/usr/sbin/lsattr -l $1 -E -F name,attribute,value,description,user_settable`;
    print $outz;
  }
}

sub Shared_Ethernet_Adapter {
  my $mac = "";
  my $vlan = "";
  my $vlan_priority = "";
  my $hostname = "";
  my $ip_addr = "";
  my $xmit_packets = 0;
  my $xmit_bytes = 0;
  my $recv_packets = 0;
  my $recv_bytes  = 0;

 foreach my $txt (`LANG=C /usr/sbin/lsdev|/usr/bin/grep 'Shared Ethernet Adapter'`) {
   $txt =~ m/([\w\d]+)\s/;
   my $ent = $1;

  foreach my $line (`LANG=C /usr/sbin/seastat -d $ent`) {
    if ($line =~ m/=========================/) {
      if ($mac ne "") {
        print $ent . "," . $mac . "," . $vlan . "," . $vlan_priority . "," . $hostname . "," . $ip_addr . "," . $xmit_packets . "," . $xmit_bytes . "," . $recv_packets . "," . $recv_bytes . "\n";
      }
      $mac = "";
      $vlan = "";
      $vlan_priority = "";
      $ip_addr = "";
      $xmit_packets = 0;
      $xmit_bytes = 0;
      $recv_packets = 0;
      $recv_bytes = 0;
      $hostname = "";
    }
    elsif($line =~ m/MAC: (.*)/) {
      $mac = $1;
    }
    elsif($line =~ m/VLAN: (.*)/) {
      $vlan = $1;
    }
    elsif($line =~ m/VLAN Priority: (.*)/) {
      $vlan_priority = $1;
    }
    elsif($line =~ m/Hostname: (.*)/) {
      $hostname = $1;
    }
    elsif($line =~ m/IP: (.*)/) {
      $ip_addr = $1;
    }
    elsif($line =~ m/Packets:\s*(\d+)\s*Packets:\s*(\d+)/) {
      $xmit_packets = $1;
      $recv_packets = $2;
    }
    elsif($line =~ m/Bytes:\s*(\d+)\s*Bytes:\s*(\d+)/) {
      $xmit_bytes = $1;
      $recv_bytes = $2;
    }
  }
 }

}

sub convert_to_hash {
  my $args = @_[0];
  my %ahash = ();
  $args =~ s/\"wwpns\=([^,]+),([^\"]+)\"/wwpn_primary\=$1\,wwpn_secondary\=$2,/;
  $args =~ s/\n//;
  while ($args =~ s/^([^\=]+)\=([^\,]+),*// ) {
    $ahash{$1} = $2;
    #print "adding $1 => $2\n";
  }
  return %ahash;
}

sub NPIV_FCP {
  my $cmd = "LANG=C /usr/ios/cli/ioscli lsnports 2>/dev/null";

  my $first = 1;
  foreach my $line (`$cmd`) {
    if ($first != 1)  {
      if ($line =~ /^(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/ ) {
      	print $1 . "," . $2 . "," . $4 . "," . $5 . "\n";
      }
    }
    $first = 0;
  }

}

sub NPIV_Mappings() {
  my $hmc = "";
  my $user = "";
  my $env    = get_env();
  my ($lpar_id, $lpar_name) = get_lslparinfo();


  my $output = `/usr/ios/cli/ioscli lssvc ITM_premium | /usr/bin/grep MANAGING_SYSTEM 2>/dev/null`;
  if ( $output =~ /^MANAGING_SYSTEM:(.*)\@(.*)/ ) {
    $hmc  = $2;
    $user = $1;
  } elsif ( $output =~ /^MANAGING_SYSTEM:(\w*)/ ) {
    $hmc  = $1;
    $user = "hscroot";
  } 

  my $new = 0;
  my %data = ();
  my $name = "";

  my %cecs        = get_hmc_cec($env, $user, $hmc, "");
  my $vio_cec = "";

  foreach my $cec (keys %cecs) {
    my $mtm_ms   = join('_', $cecs{$cec}{"type_model"}, $cecs{$cec}{"serial_num"});
    my %partitions  = get_hmc_lpars($env, $cec, $user, $hmc);
    if ( exists $partitions{$lpar_name} ) {
       $vio_cec     = $cec;
    }
  }

  if ($vio_cec eq "") {
    return;
  } 

  my $cmd = "LANG=C /usr/ios/cli/ioscli lsmap -all -npiv -fmt : -field Name Physloc ClntID ClntName ClntOS Status 'FC name' 'FC loc code' 'VFC client name' 'VFC client DRC' 2>/dev/null";
  foreach my $line (`$cmd`) {
    $line =~ s/\n//g;
    $line =~ s/ :/:/g;
    $line =~ s/: /:/g;

    my ($name, $physloc, $client_id, $client_name, $client_os, $status, $fc_name, $fc_loc_code, $vfc_client_name, $vfc_client_drc ) = split(/:/, $line);
    $data{$name}{'name'} = $name;
    $data{$name}{'physloc'} = $physloc;
    $data{$name}{'client_id'} = $client_id;
    $data{$name}{'client_os'} = $client_os;
    $data{$name}{'fc_name'} = $fc_name;
    $data{$name}{'fc_loc_code'} = $fc_loc_code;
    $data{$name}{'vfc_client_name'} = $vfc_client_name;
    $data{$name}{'vfc_client_drc'} = $vfc_client_drc;

    if ($status =~ "NOT_LOGGED_IN") {
      $data{$name}{'status'} = 0;
    }
    elsif ($status =~ "LOGGED_IN") {
      $data{$name}{'status'} = 1;
    }
    else {
      $data{$name}{'status'} = -1;
    }
  }

  my $cmd3 = ""; 
  if ($env eq "IVM") {
    $cmd3 = "/usr/ios/lpm/bin/lshwres -r virtualio --rsubtype fc -m \"$vio_cec\" --level lpar";
  } else {
    $cmd3 = "LANG=C /usr/bin/ssh $user\@$hmc lshwres -r virtualio --rsubtype fc -m \"$vio_cec\" --level lpar";
  }

  foreach my $line (`$cmd3`) {
    my %tmp_hash = convert_to_hash($line);
    foreach my $key (keys(%data)) {
      if ($data{$key}{'client_id'} == $tmp_hash{'remote_lpar_id'}) {
        my $slot = $tmp_hash{'slot_num'};
        my $physloc = $data{$key}{'physloc'};

        if ($physloc =~ /$slot$/) { 
          #found the mapping 
          $data{$key}{'client_name'} = $tmp_hash{'remote_lpar_name'};
          $data{$key}{'server_slot_number'} = $tmp_hash{'slot_num'};
          $data{$key}{'client_slot_number'} = $tmp_hash{'remote_slot_num'};
       #   $data{$key}{'client_os'} = 
        }
      }
      elsif (($data{$key}{'client_id'} == $tmp_hash{'lpar_id'})) {
        my $slot = $tmp_hash{'remote_slot_num'};
        my $physloc = $data{$key}{'physloc'};

        if ($physloc =~ /$slot$/) {
          #found the client mapping
          $data{$key}{'wwpn_primary'} = $tmp_hash{'wwpn_primary'};
          $data{$key}{'wwpn_secondary'} = $tmp_hash{'wwpn_secondary'};
        }
      }
    }
  }
  sort(%data);
  foreach my $key (sort keys(%data)) {
    print $lpar_name . "," . $lpar_id . "," . $data{$key}{'name'} . "," . 
          $data{$key}{'physloc'} . "," . $data{$key}{'client_name'} . "," . 
          $data{$key}{'client_id'} . "," . $data{$key}{'vfc_client_name'} .
          "," . $data{$key}{'client_slot_number'} . "," . 
          $data{$key}{'server_slot_number'} . "," . $data{$key}{'client_os'} .
          "," . $data{$key}{'vfc_client_drc'} . "," . $data{$key}{'fc_name'} . 
          "," . $data{$key}{'fc_loc_code'} . "," . $data{$key}{'status'} .
          "," . $data{$key}{'wwpn_primary'} . "," . $data{$key}{'wwpn_secondary'} . "\n";
  }
}



